package com.ceb.gjjs.mda.manager;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.fasterxml.jackson.annotation.*;
import com.brilliance.mda.runtime.mda.util.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.vo.CsndtlVO;

import com.ceb.gjjs.mda.manager.module.Mtabut;
import com.ceb.gjjs.mda.manager.module.Sysmod;
import com.ceb.gjjs.mda.manager.module.Csngrp;
import com.ceb.gjjs.mda.global.Platform;
import com.ceb.gjjs.mda.manager.module.Csnp;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
* 证书绑定关系详情
*/
@Transaction(vo= CsndtlVO.class)
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Csndtl extends AbstractTransaction{
		private static Logger log = LoggerFactory.getLogger(Csndtl.class);

		@BDGetter
		private Csngrp csngrp;

		@BDGetter
		private Csnp csnp;

		@BDGetter
		private Sysmod sysmod;

		@BDGetter
		private Mtabut mtabut;


		public Csndtl()
        {
            super();
            setArgument();
        }

        public void setArgument()
        {
           this.getCsnp().setCsngrpHolder(()->this.getCsngrp());
           this.getCsnp().setMtabutHolder(()->this.getMtabut());
           this.getMtabut().setMtagrpHolder(()->this.getCsngrp());
        }

        /**
        *   source:csndtl.@0001.script
        */
        @Rule(target="csnp.ame",order=100)
        public boolean ruleCsnpAmeN100(){
            IContext ctx = MdaEnv.getContext();
            
			MdaUtils.clear(this.getCsngrp().getCsn());
			String path = MdaUtils.catPath(ctx.getEnvConfig().getRootPath(),"cfg");
			String propertyPath = MdaUtils.catPath(path,"rcvsndmsg.properties");
			//String cfixbankcode = Platform.getPropertyValue(propertyPath,"OrigSender");
			this.getCsnp().setOwnref( "F" + MdaUtils.format(MdaUtils.today(),"yyMMddHHmmssSSS"));//申请序号
			log.debug("{}","ownref:"+ this.getCsnp().getOwnref());
			this.getCsngrp().getCsn().setSubtyp( "088");//报文类型
			this.getCsngrp().getCsn().setCsntyp( "3");//变更类型
			this.getCsngrp().getCsn().setCfixbankcode( MdaUtils.getPropertyValue(propertyPath,"OrigSender"));//发起节点
			this.getCsngrp().getCsn().setRem( "数字证书变更操作");
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsnp(),"ownref"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsngrp().getCsn(),"subtyp"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsngrp().getCsn(),"csntyp"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsngrp().getCsn(),"cfixbankcode"));
			

			return true;
        }

        /**
        *   source:csndtl.@0003.script
        */
        @Rule(target="csnp.shanchu",order=100)
        public boolean ruleCsnpShanchuN100(){
            IContext ctx = MdaEnv.getContext();
            
			MdaUtils.clear(this.getCsngrp().getCsn());
			String path = MdaUtils.catPath(ctx.getEnvConfig().getRootPath(),"cfg");
			String propertyPath = MdaUtils.catPath(path,"rcvsndmsg.properties");
			this.getCsnp().setOwnref( "F" + MdaUtils.format(MdaUtils.today(),"yyMMddHHmmssSSS"));//申请序号
			this.getCsngrp().getCsn().setSubtyp( "088");//报文类型
			this.getCsngrp().getCsn().setCsntyp( "2");//变更类型
			this.getCsngrp().getCsn().setCfixbankcode( MdaUtils.getPropertyValue(propertyPath,"OrigSender"));//发起节点
			this.getCsngrp().getCsn().setRem( "数字证书撤销操作");
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsnp(),"ownref"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsngrp().getCsn(),"subtyp"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsngrp().getCsn(),"csntyp"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsngrp().getCsn(),"cfixbankcode"));
			

			return true;
        }



        /**
        *   source:csndtl.@0000.script
        */
        @Init(order=100)
        public boolean	init100(){
            IContext ctx = MdaEnv.getContext();
            
			String inr = (String)ctx.restoreData("csninr");
			Platform.mdbRead(this.getCsngrp().getCsn(),new Argument<String>("inr",inr));
			this.getCsnp().setInr( inr);
			this.getCsnp().setGendat( this.getCsngrp().getCsn().getGendat());
			//$csnp\\ownref = $csngrp\\csn\\ownref;
			//log.debug("csnp ownerf:"+$csnp\\ownref.getValue());
			this.getCsnp().setSubtyp( this.getCsngrp().getCsn().getSubtyp());
			this.getCsnp().setCfixbankcode( this.getCsngrp().getCsn().getCfixbankcode());
			this.getCsnp().setRem( this.getCsngrp().getCsn().getRem());
			this.getCsnp().setSn( this.getCsngrp().getCsn().getSn());
			this.getCsnp().setCn( this.getCsngrp().getCsn().getCn());

			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsnp(),"sn"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsnp(),"cn"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsnp(),"gendat"));
			//Platform.disableDatafield($csnp\\ownref);
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsnp(),"subtyp"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsnp(),"cfixbankcode"));
			Platform.disableDatafield(MdaDriver.getDatafield(this.getCsnp(),"rem"));

			return true;
        }







}