package com.ceb.gjjs.mda.manager;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.fasterxml.jackson.annotation.*;
import com.brilliance.mda.runtime.mda.util.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.vo.DbetelVO;

import com.ceb.gjjs.mda.manager.module.Telgrp;
import com.ceb.gjjs.mda.manager.module.Mtabut;
import com.ceb.gjjs.mda.manager.module.Sysmod;
import com.ceb.gjjs.mda.manager.module.Telp;
import com.ceb.gjjs.mda.global.Platform;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
* 柜员信息修改
*/
@Transaction(vo= DbetelVO.class)
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Dbetel extends AbstractTransaction{
		private static Logger log = LoggerFactory.getLogger(Dbetel.class);

		@BDGetter
		private Telgrp telgrp;

		@BDGetter
		private Telp telp;

		@BDGetter
		private Sysmod sysmod;

		@BDGetter
		private Mtabut mtabut;


		public Dbetel()
        {
            super();
            setArgument();
        }

        public void setArgument()
        {
           this.getTelp().setTelgrpHolder(()->this.getTelgrp());
           this.getMtabut().setMtagrpHolder(()->this.getTelgrp());
        }

        /**
        *   source:dbetel.@0003.script
        */
        @Rule(target="mtabut.submit",order=1050)
        public boolean ruleMtabutSubmitN1050(){
            IContext ctx = MdaEnv.getContext();
            

			IModule usrModule = MdaDriver.getModule(this,ctx.getRoot(),"telgrp\\usr");
			IDatafield tnamField =MdaDriver.getDatafield(usrModule,"tname");
			IDatafield tchgnoField = MdaDriver.getDatafield(usrModule,"tchgno");
			IDatafield tpersoncodeField =MdaDriver.getDatafield(usrModule,"tpersoncode");
			IDatafield rolinfField = MdaDriver.getDatafield(usrModule,"rolinf");
			IDatafield tinstField =MdaDriver.getDatafield(usrModule,"tinst");
			IDatafield tchgtypField =MdaDriver.getDatafield(usrModule,"tchgtyp");
			IDatafield telField =MdaDriver.getDatafield(usrModule,"tel");
			IDatafield staField = MdaDriver.getDatafield(usrModule,"sta");
			String tel = this.getTelgrp().getUsr().getTel();

			boolean flg= true;
			if(MdaUtils.isEmpty(tnamField))
			{
			  ctx.setFieldError(tnamField,"请填写柜员姓名");
			  flg = false;
			}
			if(MdaUtils.isEmpty(tchgnoField))
			{
			  ctx.setFieldError(tchgnoField,"请填写柜员号");
			  flg = false;
			}
			if(MdaUtils.isEmpty(tpersoncodeField))
			{
			  ctx.setFieldError(tpersoncodeField,"请填写身份证");
			  flg = false;
			}
			if(MdaUtils.isEmpty(rolinfField))
			{
			  ctx.setFieldError(rolinfField,"请选择柜员角色");
			  flg = false;
			}
			if(MdaUtils.isEmpty(tinstField))
			{
			  ctx.setFieldError(tinstField,"请选择所属机构");
			  flg = false;
			}
			if(MdaUtils.isEmpty(tchgtypField))
			{
			  ctx.setFieldError(tchgtypField,"请选择变更类型");
			  flg = false;
			}
			if(MdaUtils.isEmpty(telField))
			{
			  ctx.setFieldError(telField,"请填写用户电话");
			  flg = false;
			}
			if(MdaUtils.isEmpty(staField))
			{
			  ctx.setFieldError(staField,"请选择用户状态");
			  flg = false;
			}
			if(!(tel.matches("^1(3\\d|4[5-9]|5[0-35-9]|6[567]|7[0-8]|8\\d|9[0-35-9])\\d{8}")))
			{
			  ctx.setFieldError(telField,"用户电话格式不正确");
			  flg = false;
			}
			if(!flg)
			{
			  return false;
			}

			IDatafield usrinrField = MdaDriver.getDatafield(usrModule,"inr");

			String usrinr=(String)usrinrField.getValue();
			IModule urlModule = MdaDriver.getModule(this,ctx.getRoot(),"telgrp\\rec");

			Platform.mdbUpdate(usrModule);
			ctx.getDaoSession().dbRead(urlModule,new Argument("usrinr",usrinr));
			IDatafield rolinrField = MdaDriver.getDatafield(urlModule,"rolinr");
			rolinrField.setValue((String)rolinfField.getValue());

			Platform.mdbUpdate(urlModule);


			return true;
        }



        /**
        *   source:dbetel.@0000.script
        */
        @Init(order=100)
        public boolean	init100(){
            IContext ctx = MdaEnv.getContext();
            
			String usrinr = (String)ctx.restoreData("usrinr");
			log.info("{}","获取柜员INR "+ usrinr);
			if(MdaUtils.isEmpty(usrinr))
			{
			  
			  return false;
			}
			else
			{
			  // 查询用户角色
			  String sql2= "select rolinr from url where usrinr = "+ usrinr;
			  Platform.mdbExecuteSQL(sql2);
			  Argument<String> rolinr = new  Argument<String> ("rolinr");
			  ctx.getDaoSession().dbFetchFields(rolinr);
			  String rolinrstr="";
			  if(ctx.getErrorCode()==NO_ERROR)
			  {
			    rolinrstr = rolinr.getValue();
			  }
			  Platform.mdbCloseCursor();

			  List<CodetableItem> tinstLst = new ArrayList<CodetableItem>();
			  Argument<String> tininr = new Argument<String>("inr");
			  Argument<String> tinnam = new Argument<String>("brhnam");
			  String tinsql = "select inr ,brhnam from jgx";
			  ctx.getDaoSession().dbExecuteSQL(tinsql);
			  while(true)
			  {
			    ctx.getDaoSession().dbFetchFields(tininr,tinnam);
			    int error=ctx.getErrorCode();

			    if(error==NO_ERROR)
			    {
			      tinstLst.add(new CodetableItem(tinnam.getValue(),tininr.getValue()));
			    }
			    else
			    {
			      break;
			    }

			  }
			  Platform.mdbCloseCursor();
			  //tinstLst.add(new CodetableItem("台州银行总行","11005293"));
			  //tinstLst.add(new CodetableItem("自贸区总行","1100529F"));
			  //tinstLst.add(new CodetableItem("上海自贸区分行","31005006"));
			  List<CodetableItem> sysLst = new ArrayList<CodetableItem>();
			  sysLst.add(new CodetableItem("启用","01"));
			  sysLst.add(new CodetableItem("注销","02"));
			  List<CodetableItem> tchLst = new ArrayList<CodetableItem>();
			  tchLst.add(new CodetableItem("不变","01"));
			  tchLst.add(new CodetableItem("修改","02"));
			  List<CodetableItem> thrLst = new ArrayList<CodetableItem>();
			  thrLst.add(new CodetableItem("正常","01"));
			  thrLst.add(new CodetableItem("停用","02"));
			  List<CodetableItem> staLst = new ArrayList<CodetableItem>();
			  staLst.add(new CodetableItem("正常","0"));
			  staLst.add(new CodetableItem("注销","1"));
			  staLst.add(new CodetableItem("锁定","2"));
			  // 查询角色下拉框
			  List<CodetableItem> rolLst = new ArrayList<CodetableItem>();
			  String sql="select inr,nam from rol  ";
			  Argument<String> inr=new Argument<String>("inr");
			  Argument<String> nam=new Argument<String>("nam");
			  log.info("{}","Execute SQL "+sql);
			  ctx.getDaoSession().dbExecuteSQL(sql);
			  while(true)
			  {
			    ctx.getDaoSession().dbFetchFields(inr,nam);
			    log.info("{}","Fetching "+nam.getValue());
			    int error=ctx.getErrorCode();
			    log.info("{}","Fetch complete "+error);

			    if(error==NO_ERROR)
			    {
			      rolLst.add(new CodetableItem(nam.getValue(),inr.getValue()));
			    }
			    else
			    {
			      break;
			    }
			  }
			  log.info("{}","closing cursor for usr");
			  Platform.mdbCloseCursor();
			  MdaUtils.setValues(this.getTelgrp().getUsr(),"rolinf",rolLst);
			  MdaUtils.setValues(this.getTelgrp().getUsr(),"tinst",tinstLst);
			  MdaUtils.setValues(this.getTelgrp().getUsr(),"tsyssta",sysLst);
			  MdaUtils.setValues(this.getTelgrp().getUsr(),"tchgtyp",tchLst);
			  MdaUtils.setValues(this.getTelgrp().getUsr(),"thrsta",thrLst);
			  MdaUtils.setValues(this.getTelgrp().getUsr(),"sta",staLst);

			  Platform.mdbRead(this.getTelgrp().getUsr(),new Argument<String>("inr",usrinr.toString()));
			  String tinstSting=(String)this.getTelgrp().getUsr().getTinst();
			  // tinstSting=$telp.instConvert(tinstSting);
			  log.debug("{}","tinstSting="+tinstSting);
			  this.getTelgrp().getUsr().setTinst( tinstSting);

			  String tsysstaString=(String)this.getTelgrp().getUsr().getTsyssta();
			  //tsysstaString=$telp.systemStateConvert(tsysstaString);
			  log.debug("{}","tsysstaString="+tsysstaString);
			  this.getTelgrp().getUsr().setTsyssta( tsysstaString);

			  String tchgtypString=(String)this.getTelgrp().getUsr().getTchgtyp();
			  //tchgtypString=$telp.tchgtypConvert(tchgtypString);
			  this.getTelgrp().getUsr().setTchgtyp(tchgtypString);

			  String thrstaString=(String)this.getTelgrp().getUsr().getThrsta();
			  //thrstaString=$telp.thrstaConvert(thrstaString);
			  this.getTelgrp().getUsr().setThrsta(thrstaString);

			  /* String trolesetString=(String)$telgrp\\usr\\rolinf.getValue();
			
			    if(!Platform.isEmpty(trolesetString))
			    {
			    log.debug("trolesetString="+trolesetString);
			    trolesetString=$telp.trolesetConvert(trolesetString);
			    String[] roles = trolesetString.split("\\\\|");
			    String role = roles[0];
			    String roleSql="select inr from rol where nam = ?";
			    Platform.dbExecuteSQL(roleSql,role);
			    Argument<String> roleCode = new Argument<String>("inr");
			    Platform.dbFetchFields(roleCode);
			    Platform.dbCloseCursor();
			    log.debug("trolesetString="+roleCode.getValue());
			    $telgrp\\usr\\troleset.setValue(roleCode.getValue());
			    }*/
			  this.getTelgrp().getUsr().setRolinf(rolinrstr);
			  
			}


			return true;
        }







}