package com.ceb.gjjs.mda.manager;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.fasterxml.jackson.annotation.*;
import com.brilliance.mda.runtime.mda.util.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.vo.FmtselVO;

import com.ceb.gjjs.mda.manager.module.Sysmod;
import com.ceb.gjjs.mda.manager.module.Fmpp;
import com.ceb.gjjs.mda.global.Platform;
import com.ceb.gjjs.mda.manager.module.Fmtgrp;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
* 境内外币报文查询
*/
@Transaction(vo= FmtselVO.class)
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Fmtsel extends AbstractTransaction{
		private static Logger log = LoggerFactory.getLogger(Fmtsel.class);

		@BDGetter
		private Fmtgrp fmtgrp;

		@BDGetter
		private Sysmod sysmod;

		@BDGetter
		private Fmpp fmpp;


		public Fmtsel()
        {
            super();
            setArgument();
        }

        public void setArgument()
        {
           this.getFmpp().setFmtgrpHolder(()->this.getFmtgrp());
        }

        /**
        *   source:fmtsel.@0000.script
        */
        @Rule(target="fmpp.sel",order=100)
        public boolean ruleFmppSelN100(){
            IContext ctx = MdaEnv.getContext();
            
			MdaUtils.clear(this.getFmtgrp().getRcvlst());
			String sql = "WHERE m1.inr = m2.inr AND m1.pntinr IS NULL";

			List sqldata = new ArrayList();

			//起始时间
			if(!MdaUtils.isEmpty(this.getFmpp().getStatim()))
			{
			  sql = sql + " AND m1.rcvdattim>=?";
			  sqldata.add(this.getFmpp().getStatim());
			}
			//截止时间
			if(!MdaUtils.isEmpty(this.getFmpp().getEndtim()))
			{
			  sql = sql + " AND m1.rcvdattim<=?";
			  sqldata.add(this.getFmpp().getEndtim());
			}
			//报文标准
			if(!MdaUtils.isEmpty(this.getFmpp().getMsgtyp()))
			{
			  sql = sql+" AND m2.msgtyp=?";
			  sqldata.add(this.getFmpp().getMsgtyp());
			}
			//报文类型
			if(!MdaUtils.isEmpty(this.getFmpp().getSubtyp()))
			{
			  sql = sql+" AND m2.subtyp=?";
			  sqldata.add(this.getFmpp().getSubtyp());
			}
			//20域编号
			if(!MdaUtils.isEmpty(this.getFmpp().getOthref()))
			{
			  sql = sql+" AND m2.othref LIKE'"+"%"+this.getFmpp().getOthref()+"%"+"'";
			}
			//21域编号
			if(!MdaUtils.isEmpty(this.getFmpp().getOwnref()))
			{
			  sql = sql+" AND m2.ownref LIKE'"+"%"+this.getFmpp().getOwnref()+"%"+"'";
			}
			//收发方向
			if(!MdaUtils.isEmpty(this.getFmpp().getDir()))
			{
			  sql = sql+" AND m1.dir=?";
			  sqldata.add(this.getFmpp().getDir());
			}
			//币种
			if(!MdaUtils.isEmpty(this.getFmpp().getCur()))
			{
			  sql = sql+" AND m2.cur=?";
			  sqldata.add(this.getFmpp().getCur());
			}
			//帐号
			if(!MdaUtils.isEmpty(this.getFmpp().getAct()))
			{
			  sql = sql+" AND m2.act LIKE'"+"%"+this.getFmpp().getAct()+"%"+"'";
			}
			//金额
			if(!MdaUtils.isEmpty(this.getFmpp().getAmtsta()))
			{
			  sql = sql+" AND m2.amt>=?";
			  sqldata.add(this.getFmpp().getAmtsta());
			}
			if(!MdaUtils.isEmpty(this.getFmpp().getAmtend()))
			{
			  sql = sql+" AND m2.amt<=?";
			  sqldata.add(this.getFmpp().getAmtend());
			}
			//对账状态
			if(!MdaUtils.isEmpty(this.getFmpp().getActsta()))
			{
			  sql = sql+" AND m2.actsta=?";
			  sqldata.add(this.getFmpp().getActsta());
			}

			String params = Platform.sgetTransParam();
			if(!MdaUtils.isEmpty(params)&&"sig".equals(params))//数字签名管理
			{
			  sql = sql +" AND ( m2.subtyp in ('023','024') or (m1.sta='D' AND m1.flg='1') ) ";
			}
			else  if(!MdaUtils.isEmpty(params)&&"acc".equals(params))//业务对账
			{
			  sql = sql +" AND m2.subtyp in ('100','101','102','103','200','201','202','203','204','014') ";
			}
			else
			{
			  //验签标志
			  if(!MdaUtils.isEmpty(this.getFmpp().getFlg()))
			  {
			    sql = sql+" AND m1.flg=?";
			    sqldata.add(this.getFmpp().getFlg());
			  }
			  //收发状态
			  if(!MdaUtils.isEmpty(this.getFmpp().getSta()))
			  {
			    sql = sql+" AND m1.sta=?";
			    sqldata.add(this.getFmpp().getSta());
			  }
			}
			sql = sql+" ORDER BY m1.inr ASC";
			log.debug("{}","sql："+sql);
			Platform.mdbReadset(new IModuleList[]{this.getFmtgrp().getMpss(),this.getFmtgrp().getMpis()},0,sql,sqldata.toArray());
			//组合成为rcvlst
			if(ctx.getErrorCode()==NO_ERROR)
			{
			  for(int i=0;i<this.getFmtgrp().getMpss().fullSize();i++)
			  {
			    if("<".equals(this.getFmtgrp().getMpss().getElement(i).getDir()))
			    {
			      //子系统和行内系统置位
			      this.getFmtgrp().getMpss().getElement(i).setChncod( this.getFmtgrp().getMpss().getElement(i).getSrcchncod());
			      if(!MdaUtils.isEmpty(this.getFmtgrp().getMpss().getElement(i).getNxtchncod()))
			      {
			        Platform.mdbRead(this.getFmpp().getChnxor(),new Argument<String>("cod",EQ,this.getFmtgrp().getMpss().getElement(i).getNxtchncod()));
			        if(!"N".equals(this.getFmpp().getChnxor().getTyp()))
			        {
			          this.getFmtgrp().getMpss().getElement(i).setNxtchncod(NULLSTR);
			        }
			      }
			    }
			    else if(">".equals(this.getFmtgrp().getMpss().getElement(i).getDir()))
			    {
			      //子系统和行内系统置位
			      if(!MdaUtils.isEmpty(this.getFmtgrp().getMpss().getElement(i).getNxtchncod()))
			      {
			        Platform.mdbRead(this.getFmpp().getChnxor(),new Argument<String>("cod",EQ,this.getFmtgrp().getMpss().getElement(i).getNxtchncod()));
			        if("B".equals(this.getFmpp().getChnxor().getTyp()))
			        {
			          this.getFmtgrp().getMpss().getElement(i).setChncod( this.getFmtgrp().getMpss().getElement(i).getNxtchncod());
			          this.getFmtgrp().getMpss().getElement(i).setNxtchncod(NULLSTR);
			        }
			        else if("N".equals(this.getFmpp().getChnxor().getTyp()))
			        {
			          this.getFmtgrp().getMpss().getElement(i).setChncod(NULLSTR);
			        }
			        else
			        {
			          this.getFmtgrp().getMpss().getElement(i).setChncod(NULLSTR);
			          this.getFmtgrp().getMpss().getElement(i).setNxtchncod(NULLSTR);
			        }
			      }
			    }
			    this.getFmtgrp().getRcvlst().add(this.getFmtgrp().getMpss().getElement(i).getInr(),this.getFmtgrp().getMpss().getElement(i).getDir(),this.getFmtgrp().getMpss().getElement(i).getSrcchncod(),this.getFmtgrp().getMpss().getElement(i).getChncod(),this.getFmtgrp().getMpss().getElement(i).getNxtchncod(),this.getFmtgrp().getMpss().getElement(i).getTgtchncod(),this.getFmtgrp().getMpss().getElement(i).getSmhinr(),this.getFmtgrp().getMpss().getElement(i).getSta(),this.getFmtgrp().getMpss().getElement(i).getRspsta(),this.getFmtgrp().getMpss().getElement(i).getRcvdattim(),this.getFmtgrp().getMpss().getElement(i).getLstdattim(),this.getFmtgrp().getMpss().getElement(i).getSttdattim(),this.getFmtgrp().getMpss().getElement(i).getTgtdattime(),this.getFmtgrp().getMpss().getElement(i).getSnddattim(),this.getFmtgrp().getMpss().getElement(i).getRspdattim(),this.getFmtgrp().getMpis().getElement(i).getOwnref(),this.getFmtgrp().getMpis().getElement(i).getOthref(),this.getFmtgrp().getMpis().getElement(i).getSndbic(),this.getFmtgrp().getMpis().getElement(i).getRcvbic(),this.getFmtgrp().getMpis().getElement(i).getAct(),this.getFmtgrp().getMpis().getElement(i).getDtlchg(),this.getFmtgrp().getMpis().getElement(i).getCur(),this.getFmtgrp().getMpis().getElement(i).getAmt(),this.getFmtgrp().getMpis().getElement(i).getValdat(),this.getFmtgrp().getMpis().getElement(i).getMsgtyp(),this.getFmtgrp().getMpis().getElement(i).getSubtyp(),this.getFmtgrp().getMpis().getElement(i).getActbic(),this.getFmtgrp().getMpis().getElement(i).getT53bic(),this.getFmtgrp().getMpis().getElement(i).getUsrnam(),this.getFmtgrp().getMpis().getElement(i).getActsta(),this.getFmtgrp().getMpss().getElement(i).getFlg());
			  }
			}
			log.debug("{}","fmtsel查出rcvlst记录"+this.getFmtgrp().getRcvlst().fullSize());
			
			if(this.getFmtgrp().getRcvlst().fullSize() == 0)
			{
			  ctx.setMessage(MdaUtils.getI18NString("fmtsel","CT000000"),MdaUtils.getI18NString("fmtsel","CT000001"));
			}

			return true;
        }

        /**
        *   source:fmtsel.@0001.script
        */
        @Rule(target="fmpp.clear",order=100)
        public boolean ruleFmppClearN100(){
            IContext ctx = MdaEnv.getContext();
            
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"statim"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"endtim"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"ownref"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"othref"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"dir"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"subtyp"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"cur"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"act"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"amtsta"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"amtend"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"flg"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"sta"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getFmpp(),"actsta"));

			return true;
        }

        /**
        *   source:fmtsel.@0002.script
        */
        @Rule(target="fmtgrp.rcvlst",order=100)
        public boolean ruleFmtgrpRcvlstN100(){
            IContext ctx = MdaEnv.getContext();
            
			int index = 0;
			if(index >-1)
			{
			  String inr = this.getFmtgrp().getRcvlst().getElement(index).getInr();
			  ctx.storeData("inr",inr);//mpsinr
			  Platform.mchain("fmtdtl");
			}

			return true;
        }

        /**
        *   source:fmtsel.@0006.script
        */
        @Rule(target="fmpp.rtn",order=100)
        public boolean ruleFmppRtnN100(){
            IContext ctx = MdaEnv.getContext();
            
			

			return true;
        }

        /**
        *   source:fmtsel.@0007.script
        */
        @Rule(target="fmpp.exp",order=100)
        public boolean ruleFmppExpN100(){
            IContext ctx = MdaEnv.getContext();
            
			IStream is = this.getFmpp().getExp();
			is.setType("xls");
			is.setName("Cfc"+MdaUtils.format(MdaUtils.today(),"yyyyMMddHHmmss")+".xls");
			List<Object[]> lst = new ArrayList<Object[]>();
			String [] title= new String[17];
			title[0]=MdaUtils.getI18NString("fmtsel","CT000019");
			title[1]=MdaUtils.getI18NString("fmtsel","CT000020");
			title[2]=MdaUtils.getI18NString("fmtsel","CT000021");
			title[3]=MdaUtils.getI18NString("fmtsel","CT000022");
			title[4]=MdaUtils.getI18NString("fmtsel","CT000023");
			title[5]=MdaUtils.getI18NString("fmtsel","CT000024");
			title[6]=MdaUtils.getI18NString("fmtsel","CT000025");
			title[7]=MdaUtils.getI18NString("fmtsel","CT000026");
			title[8]=MdaUtils.getI18NString("fmtsel","CT000027");
			title[9]=MdaUtils.getI18NString("fmtsel","CT000028");
			title[10]=MdaUtils.getI18NString("fmtsel","CT000029");
			title[11]=MdaUtils.getI18NString("fmtsel","CT000030");
			title[12]=MdaUtils.getI18NString("fmtsel","CT000031");
			title[13]=MdaUtils.getI18NString("fmtsel","CT000032");
			title[14]=MdaUtils.getI18NString("fmtsel","CT000033");
			title[15]=MdaUtils.getI18NString("fmtsel","CT000034");
			title[16]=MdaUtils.getI18NString("fmtsel","CT000035");
			lst.add(title);
			log.debug("{}","step1");
			for(int i=0;i<this.getFmtgrp().getRcvlst().fullSize();i++)
			{
			  Object [] line = new Object[17];
			  Date vludat = this.getFmtgrp().getRcvlst().getElement(i).getValdat();
			  line[0]=MdaUtils.format(vludat,"yyyy-MM-dd");
			  line[1]=this.getFmtgrp().getRcvlst().getElement(i).getOthref();
			  line[2]=this.getFmtgrp().getRcvlst().getElement(i).getOwnref();
			  line[3]=this.getFmtgrp().getRcvlst().getElement(i).getCur();
			  BigDecimal amt = this.getFmtgrp().getRcvlst().getElement(i).getAmt();
			  amt = amt.setScale(2);
			  line[4]=amt.toString();
			  line[5]=this.getFmtgrp().getRcvlst().getElement(i).getAct();
			  line[6]=this.getFmtgrp().getRcvlst().getElement(i).getDtlchg();
			  String nxtchncod = this.getFmtgrp().getRcvlst().getElement(i).getNxtchncod();
			  if(MdaUtils.isEmpty(nxtchncod))
			    line[7]="";
			  else
			    line[7]=MdaUtils.getCodetableLabel("chncod",nxtchncod);
			  String chncod = this.getFmtgrp().getRcvlst().getElement(i).getChncod();
			  if(MdaUtils.isEmpty(chncod))
			    line[8]="";
			  else
			    line[8]=MdaUtils.getCodetableLabel("chncod",chncod);
			  line[9]=this.getFmtgrp().getRcvlst().getElement(i).getT53bic();
			  line[10]=this.getFmtgrp().getRcvlst().getElement(i).getActbic();
			  line[11]=this.getFmtgrp().getRcvlst().getElement(i).getSndbic();
			  line[12]=this.getFmtgrp().getRcvlst().getElement(i).getRcvbic();
			  String msgtyp = this.getFmtgrp().getRcvlst().getElement(i).getMsgtyp();
			  line[13]=MdaUtils.getCodetableLabel("msgmst",msgtyp);
			  line[14]=this.getFmtgrp().getRcvlst().getElement(i).getSubtyp();
			  Date rcvdattim =this.getFmtgrp().getRcvlst().getElement(i).getRcvdattim();
			  line[15]=MdaUtils.format(rcvdattim,"yyyy-MM-dd HH:mm:ss");
			  String sta = this.getFmtgrp().getRcvlst().getElement(i).getSta();
			  line[16]=MdaUtils.getCodetableLabel("msgsta",sta);
			  lst.add(line);
			}
			MdaUtils.unloadExcel(lst,is,false);

			return true;
        }


        /**
        *   source:fmtsel.@0009.script
        */
        @Check(target="fmpp.ownref",order=100)
        public boolean checkFmppOwnrefN100(){
            IContext ctx = MdaEnv.getContext();
            
			Platform.transAndCheckRef(MdaDriver.getDatafield(this.getFmpp(),"ownref"));

			return true;
        }

        /**
        *   source:fmtsel.@0010.script
        */
        @Check(target="fmpp.othref",order=100)
        public boolean checkFmppOthrefN100(){
            IContext ctx = MdaEnv.getContext();
            
			Platform.transAndCheckRef(MdaDriver.getDatafield(this.getFmpp(),"othref"));

			return true;
        }


        /**
        *   source:fmtsel.@0003.script
        */
        @Init(order=100)
        public boolean	init100(){
            IContext ctx = MdaEnv.getContext();
            
			if(!MdaUtils.isReturnCall())
			{
			  this.getFmpp().setStatim( Platform.formatDateTime(MdaUtils.add(MdaUtils.today(),-3),"0"));
			  this.getFmpp().setEndtim( Platform.formatDateTime(MdaUtils.today(),"235959"));

			  MdaUtils.setValues(this.getFmpp(),"cur",this.getSysCurList());

			  List<CodetableItem> flst = new ArrayList<CodetableItem>();
			  flst.add(new CodetableItem("验签成功","0"));
			  flst.add(new CodetableItem("验签失败","1"));
			  MdaUtils.setValues(this.getFmpp(),"flg",flst);

			  List<CodetableItem> mlst = new ArrayList<CodetableItem>();
			  mlst.add(new CodetableItem("MT","sf2"));
			  mlst.add(new CodetableItem("FMT","txt"));
			  MdaUtils.setValues(this.getFmpp(),"msgtyp",mlst);
			  this.getFmpp().setMsgtyp( "txt");
			  
			  
			  MdaUtils.setValues(this.getFmpp(),"subtyp",getFmtList());
			  List<CodetableItem> slst = new ArrayList<CodetableItem>();
			  slst.add(new CodetableItem("已对账","1"));
			  slst.add(new CodetableItem("未对账",""));
			  MdaUtils.setValues(this.getFmpp(),"actsta",slst);
			  this.getFmpp().setLbl( "对账状态：");

			  String params = Platform.sgetTransParam();
			  if(!MdaUtils.isEmpty(params))
			  {
			    if("chk".equals(params))//报文复核
			    {
			      //Platform.setValues($fmpp\\subtyp,getFaaList());
			      this.getFmpp().setSta( "V");
			      
			      
			      
			    }
			    else if("sig".equals(params))//签名查询查复
			    {
			      //$fmpp\\sta = "D";
			      //$fmpp\\flg = "1";//验签失败
			      
			      
			      
			      
			    }
			    else if("acc".equals(params))//业务对账
			    {
			      MdaUtils.setValues(this.getFmpp(),"subtyp",getFmsList());
			    }
			  }
			}


			return true;
        }


        /**
        *   source:fmtsel.@0004.script
        */
        public List<CodetableItem> getFmsList()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			//得到所有报文类型名称
			List<CodetableItem> lst = new ArrayList<CodetableItem>();
			lst.add(new CodetableItem("100","100"));
			lst.add(new CodetableItem("101","101"));
			lst.add(new CodetableItem("102","102"));
			lst.add(new CodetableItem("103","103"));
			lst.add(new CodetableItem("104","104"));
			lst.add(new CodetableItem("200","200"));
			lst.add(new CodetableItem("201","201"));
			lst.add(new CodetableItem("202","202"));
			lst.add(new CodetableItem("204","204"));
			lst.add(new CodetableItem("014","014"));
			return lst;
		}
        /**
        *   source:fmtsel.@0005.script
        */
        public List<CodetableItem> getFaaList()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			//得到所有报文类型名称
			List<CodetableItem> mtyList = new ArrayList<CodetableItem>();//构造码表mtyList
			Platform.mdbExecuteSQL("SELECT subtyp FROM fmty WHERE  flg=1 ORDER BY subtyp");//执行sql语句查询记录
			Argument<String> nam = new Argument<String>("subtyp");//构造Label列nam
			while(true)
			{
			  Platform.mdbFetchFields(nam);//逐条取出记录
			  int errorCode = ctx.getErrorCode();//取出记录的执行结果
			  if(errorCode == NO_ERROR)
			  {
			    String mtyval = nam.value;
			    if(!MdaUtils.isEmpty(mtyval))
			    {
			      mtyList.add(new CodetableItem(mtyval,mtyval));
			    }
			  }
			  else
			  {
			    break;
			  }
			}
			Platform.mdbCloseCursor();
			return mtyList;
		}
        /**
        *   source:fmtsel.@0008.script
        */
        public List<CodetableItem> getFmtList()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			//得到所有报文类型名称
			List<CodetableItem> mtyList = new ArrayList<CodetableItem>();//构造码表mtyList
			Platform.mdbExecuteSQL("SELECT mty FROM mfd WHERE tno=1 AND mty like 'FMT%' ORDER BY mty");//执行sql语句查询记录
			Argument<String> nam = new Argument<String>("mty");//构造Label列nam
			while(true)
			{
			  Platform.mdbFetchFields(nam);//逐条取出记录
			  int errorCode = ctx.getErrorCode();//取出记录的执行结果
			  if(errorCode == NO_ERROR)
			  {
			    String mtyval = nam.value;
			    if(!MdaUtils.isEmpty(mtyval)&&mtyval.length()>5)
			    {
			      mtyval = mtyval.substring(4);
			      mtyList.add(new CodetableItem(mtyval,mtyval));
			    }
			  }
			  else
			  {
			    break;
			  }
			}
			Platform.mdbCloseCursor();
			return mtyList;
		}
        /**
        *   source:fmtsel.@0011.script
        */
        public List<CodetableItem> getSysCurList()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			List<CodetableItem> curLst = new ArrayList<CodetableItem>();
			String sql = "SELECT cur,curnam FROM cur";
			Platform.mdbExecuteSQL(sql);
			Argument<String> cur = new Argument<String>("cur");
			Argument<String> curnam = new Argument<String>("curnam");
			while(true)
			{
			  Platform.mdbFetchFields(cur,curnam);
			  if(ctx.getErrorCode()==NO_ERROR)
			  {
			    if(curnam.value != null && cur.value != null)
			    {
			      CodetableItem c = new CodetableItem(cur.value,cur.value);
			      curLst.add(c);
			    }
			  }
			  else
			    break;
			}
			Platform.mdbCloseCursor();
			return curLst;
		}




}