package com.ceb.gjjs.mda.manager;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.fasterxml.jackson.annotation.*;
import com.brilliance.mda.runtime.mda.util.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.vo.PbtselVO;

import com.ceb.gjjs.mda.manager.module.Sysmod;
import com.ceb.gjjs.mda.global.Platform;
import com.ceb.gjjs.mda.manager.module.Pbtp;
import com.ceb.gjjs.mda.manager.module.Pbtgrp;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
* 间接参与者行号信息处理交易
*/
@Transaction(vo= PbtselVO.class)
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Pbtsel extends AbstractTransaction{
		private static Logger log = LoggerFactory.getLogger(Pbtsel.class);

		@BDGetter
		private Pbtgrp pbtgrp;

		@BDGetter
		private Pbtp pbtp;

		@BDGetter
		private Sysmod sysmod;


		public Pbtsel()
        {
            super();
            setArgument();
        }

        public void setArgument()
        {
           this.getPbtp().setPbtgrpHolder(()->this.getPbtgrp());
        }

        /**
        *   source:pbtsel.@0000.script
        */
        @Rule(target="pbtp.add",order=100)
        public boolean rulePbtpAddN100(){
            IContext ctx = MdaEnv.getContext();
            
			Platform.mchain("pbtadd");

			return true;
        }

        /**
        *   source:pbtsel.@0001.script
        */
        @Rule(target="pbtp.ret",order=100)
        public boolean rulePbtpRetN100(){
            IContext ctx = MdaEnv.getContext();
            
			

			return true;
        }

        /**
        *   source:pbtsel.@0002.script
        */
        @Rule(target="pbtp.sel",order=100)
        public boolean rulePbtpSelN100(){
            IContext ctx = MdaEnv.getContext();
            
			this.query();

			return true;
        }

        /**
        *   source:pbtsel.@0004.script
        */
        @Rule(target="pbtp.del",order=100)
        public boolean rulePbtpDelN100(){
            IContext ctx = MdaEnv.getContext();
            
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"bnkbic"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"bnknam"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"bnktyp"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"nam"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"tel"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"zipcod"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"email"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"pvccod"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"ctycod"));
			Platform.clearDatafield(MdaDriver.getDatafield(this.getPbtp(),"ptycod"));

			return true;
        }

        /**
        *   source:pbtsel.@0005.script
        */
        @Rule(target="pbtgrp.bnklst",order=100)
        public boolean rulePbtgrpBnklstN100(){
            IContext ctx = MdaEnv.getContext();
            
			int index = 0;
			if(index>-1)
			{
			  String pbtinr = this.getPbtgrp().getBnklst().getElement(index).getInr();
			  ctx.storeData("pbtinr",pbtinr);
			  Platform.mchain("pbtdtl");
			}

			return true;
        }

        /**
        *   source:pbtsel.@0007.script
        */
        @Rule(target="pbtp.exp",order=100)
        public boolean rulePbtpExpN100(){
            IContext ctx = MdaEnv.getContext();
            
			IStream is = this.getPbtp().getExp();
			is.setType("xls");
			is.setName("Pbt"+MdaUtils.format(MdaUtils.today(),"yyyyMMddHHmmss")+".xls");
			List<Object[]> lst = new ArrayList<Object[]>();
			String [] title= new String[16];
			title[0]=MdaUtils.getI18NString("pbtsel","CT000086");
			title[1]=MdaUtils.getI18NString("pbtsel","CT000087");
			title[2]=MdaUtils.getI18NString("pbtsel","CT000088");
			title[3]=MdaUtils.getI18NString("pbtsel","CT000089");
			title[4]=MdaUtils.getI18NString("pbtsel","CT000090");
			title[5]=MdaUtils.getI18NString("pbtsel","CT000091");
			title[6]=MdaUtils.getI18NString("pbtsel","CT000092");
			title[7]=MdaUtils.getI18NString("pbtsel","CT000093");
			title[8]=MdaUtils.getI18NString("pbtsel","CT000094");
			title[9]=MdaUtils.getI18NString("pbtsel","CT000095");
			title[10]=MdaUtils.getI18NString("pbtsel","CT000096");
			title[11]=MdaUtils.getI18NString("pbtsel","CT000097");
			title[12]=MdaUtils.getI18NString("pbtsel","CT000098");
			title[13]=MdaUtils.getI18NString("pbtsel","CT000099");
			title[14]=MdaUtils.getI18NString("pbtsel","CT000100");
			title[15]=MdaUtils.getI18NString("pbtsel","CT000101");
			lst.add(title);
			for(int i=0;i<this.getPbtgrp().getBnklst().fullSize();i++)
			{
			  Object [] line = new Object[16];
			  String typ = this.getPbtgrp().getBnklst().getElement(i).getBnktyp();
			  if(MdaUtils.isEmpty(typ))
			    line[0] = "";
			  else
			    line[0] = MdaUtils.getCodetableLabel("bnktyp",typ);
			  line[1] = this.getPbtgrp().getBnklst().getElement(i).getBic();
			  line[2] = this.getPbtgrp().getBnklst().getElement(i).getDirectbic();
			  line[3] = this.getPbtgrp().getBnklst().getElement(i).getHafnam();
			  line[4] = this.getPbtgrp().getBnklst().getElement(i).getFulnam();
			  line[5] = this.getPbtgrp().getBnklst().getElement(i).getProvcod();
			  line[6] = this.getPbtgrp().getBnklst().getElement(i).getCitycod();
			  line[7] = this.getPbtgrp().getBnklst().getElement(i).getBnkcod();
			  line[8] = this.getPbtgrp().getBnklst().getElement(i).getLnkman();
			  line[9] = this.getPbtgrp().getBnklst().getElement(i).getPhone();
			  line[10] = this.getPbtgrp().getBnklst().getElement(i).getEmail();
			  Date begdat = this.getPbtgrp().getBnklst().getElement(i).getBegdat();
			  line[11] = MdaUtils.format(begdat,"yyyy/MM/dd");
			  Date enddat = this.getPbtgrp().getBnklst().getElement(i).getLosdat();
			  line[12] = MdaUtils.format(enddat,"yyyy/MM/dd");
			  line[13] = this.getPbtgrp().getBnklst().getElement(i).getChangeterm();
			  String chnsta = this.getPbtgrp().getBnklst().getElement(i).getLstupdate();
			  if(MdaUtils.isEmpty(chnsta))
			    line[14] = "";
			  else
			    line[14] = MdaUtils.getCodetableLabel("csntyp",chnsta);

			  String sta = this.getPbtgrp().getBnklst().getElement(i).getSta();
			  if(MdaUtils.isEmpty(sta))
			    line[15] = "";
			  else
			    line[15] = MdaUtils.getCodetableLabel("bnksta",sta);
			  lst.add(line);
			}
			MdaUtils.unloadExcel(lst,is,false);

			return true;
        }


        /**
        *   source:pbtsel.@0008.script
        */
        @Check(target="pbtp.bnkbic",order=100)
        public boolean checkPbtpBnkbicN100(){
            IContext ctx = MdaEnv.getContext();
            
			if(MdaUtils.isEmpty(this.getPbtp().getBnkbic()))
			  return true;
			String sndbak = this.getPbtp().getBnkbic();
			sndbak = sndbak.toUpperCase();
			if(!sndbak.matches("[A-Z0-9]+"))
			{
			  ctx.setFieldError(this,"pbtp.bnkbic",MdaUtils.getI18NString("pbtsel","CT000111"));
			}
			this.getPbtp().setBnkbic( sndbak);
			

			return true;
        }


        /**
        *   source:pbtsel.@0006.script
        */
        @Init(order=200)
        public boolean	init200(){
            IContext ctx = MdaEnv.getContext();
            
			MdaUtils.setValues(this.getPbtp(),"pvccod",this.getPbtp().getProcodList());
			
			/*String ptyp = "";
			  if(!Platform.isEmpty($pbtp\\pvccod))
			  {
			  ptyp = $pbtp\\pvccod;
			  }
			  log.debug("pvccod: " + ptyp);
			  Platform.setValues($pbtp\\ctycod,$pbtp.getCtycodList(ptyp));
			  Platform.setModified($pbtp\\ctycod);
			  String typ = "";
			  if(!Platform.isEmpty($pbtp\\ctycod))
			  {
			  typ = $pbtp\\ctycod;
			  }
			  log.debug("ctycod: " + typ);
			  Platform.setValues($pbtp\\ptycod,$pbtp.getPtycodList(typ,0));
			  Platform.setModified($pbtp\\ptycod);
			  */

			return true;
        }


        /**
        *   source:pbtsel.@0003.script
        */
        public void query()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			log.debug("{}","@@@@ 行号信息查询 @@@@");
			Argument arg = new Argument();
			String[] nams = {"bnktyp","provcod","citycod","bnkcod"};
			arg = Platform.addArgumentNotEmpty(nams,MdaDriver.getDatafield(this.getPbtp(),"bnktyp"),MdaDriver.getDatafield(this.getPbtp(),"pvccod"),MdaDriver.getDatafield(this.getPbtp(),"ctycod"),MdaDriver.getDatafield(this.getPbtp(),"ptycod"));
			String[] likenams = {"bic","hafnam","lnkman","phone"};
			Argument argLike = Platform.addStringArgumentOfLIKE(likenams,MdaDriver.getDatafield(this.getPbtp(),"bnkbic"),MdaDriver.getDatafield(this.getPbtp(),"bnknam"),MdaDriver.getDatafield(this.getPbtp(),"nam"),MdaDriver.getDatafield(this.getPbtp(),"tel"));
			arg = arg.and(argLike);
			arg = arg.and(new Argument("losdat",GT,MdaUtils.sqlDate(MdaUtils.today(),GT)));
			arg = arg.and(new Argument("begdat",LE,MdaUtils.sqlDate(MdaUtils.today(),LE)));

			log.debug("{}","arg: " + arg.value);
			Platform.mdbReadset(this.getPbtgrp().getBnklst(),0,arg,new Argument<String>("inr",DESC));
			
			if(this.getPbtgrp().getBnklst().size()==0)
			{
			  ctx.setMessage(MdaUtils.getI18NString("pbtsel","CT000020"),MdaUtils.getI18NString("pbtsel","CT000021"));
			}
			log.debug("{}","@@@@ 行号信息查询完成 @@@@");
		}




}