package com.ceb.gjjs.mda.manager.module;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.brilliance.mda.runtime.mda.util.*;
import com.fasterxml.jackson.annotation.*;
import java.util.regex.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.global.Platform;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
* 
*/
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Cfxp extends AbstractModule{
		private static Logger log = LoggerFactory.getLogger(Cfxp.class);

        //发起机构行号
        private String instgpty;

        //接收机构行号
        private String instdpty;

        //报文发起时间
        private String snddttm;

        //报文标识号
        private String msgid;

        //报文参考号
        private String ref;

        //信息内容
        private String msgcntt;

        //发送
        private IControl sndbtn;

        //清除
        private IControl clrbtn;


        @JsonIgnore
        @BDGetter
        private transient IPanel sndpnl;


	    @BDGetter
		private Msgmod msgmod;

	    @BDGetter
		private Smpmod smpmod;


		public Cfxp()
        {
            super();
            setArgument();
        }

        public Cfxp(String name,IModule parent)
        {
            super(name,parent);
            setArgument();
        }

        public void setArgument()
        {
        }

        /**
        *   source:cfxp.@0000.script
        */
        @Rule(target="sndbtn",order=100)
        public boolean ruleSndbtnN100(){
            IContext ctx = MdaEnv.getContext();
            
			// 校验必填项
			if(!ctx.checkAll(this))
			{
			  ctx.setMessage(MdaUtils.getI18NString("cfxp","CT000009"),MdaUtils.getI18NString("cfxp","CT000010"));
			  return false;
			}
			String json998 = this.getMsgmod().create_msg_998((String)this.getInstgpty(),(String)this.getInstdpty(),(String)this.getMsgid(),(String)this.getRef(),this.getMsgcntt());
			// 保存报文
			if(!this.storeMsg(json998))
			{
			  ctx.setMessage(MdaUtils.getI18NString("cfxp","CT000011"),MdaUtils.getI18NString("cfxp","CT000012"));
			  ctx.getDaoSession().dbRollback();
			  return false;
			}
			else
			{
			  log.info("{}","自由格式报文发送成功");
			  ctx.setMessage(MdaUtils.getI18NString("cfxp","CT000072"),MdaUtils.getI18NString("cfxp","CT000073"));
			}


			return true;
        }

        /**
        *   source:cfxp.@0003.script
        */
        @Rule(target="clrbtn",order=100)
        public boolean ruleClrbtnN100(){
            IContext ctx = MdaEnv.getContext();
            
			// 清空已填写的内容
			this.setInstdpty(NULLSTR);
			this.setRef(NULLSTR);
			this.setMsgcntt(NULLSTR);
			

			return true;
        }


        /**
        *   source:cfxp.@0005.script
        */
        @Check(target="instdpty",order=100)
        public boolean checkInstdptyN100(){
            IContext ctx = MdaEnv.getContext();
            
			if(MdaUtils.isEmpty(this.getInstdpty()))
			{
			  ctx.setFieldError(this,"instdpty",MdaUtils.getI18NString("cfxp","CT000048"));
			  return false;
			}

			return true;
        }

        /**
        *   source:cfxp.@0006.script
        */
        @Check(target="msgcntt",order=100)
        public boolean checkMsgcnttN100(){
            IContext ctx = MdaEnv.getContext();
            
			if(MdaUtils.isEmpty(this.getMsgcntt()))
			{
			  ctx.setFieldError(this,"msgcntt",MdaUtils.getI18NString("cfxp","CT000051"));
			  return false;
			}

			return true;
        }


        /**
        *   source:cfxp.@0002.script
        */
        @Init(order=100)
        public boolean	init100(){
            IContext ctx = MdaEnv.getContext();
            
			// 初始化页面参数
			// 发起机构行号,不可编辑
			this.setInstgpty( "TZBKCNBT");
			// 报文标识符，不可编辑
			this.setMsgid( this.getMsgmod().mkMsgId());


			return true;
        }


        /**
        *   source:cfxp.@0001.script
        */
        public boolean storeMsg(Object obj)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			//存储报文内容至指定目录，并生成mps，mpi，smh信息
			MdaUtils.clear(this.getSmpmod().getFilstm());
			IStream is = this.getSmpmod().getFilstm();
			String str = (String)obj;
			try
			{
			  is.getOutputStream().write(str.getBytes("utf-8"));//将报文存入流
			}
			catch(java.io.IOException ee)
			{
			  log.info("{}","加载报文信息失败");
			  return false;
			}
			if(!this.getSmpmod().createMessageProcessSubject(is,"HWE","json","CPX","系统管理报文"))
			{
			  log.debug("{}","报文保存失败");
			  return false;
			}
			else
			{
			  // Platform.message(#CT000025,#CT000026);
			  return true;
			}
		}



















}