package com.ceb.gjjs.mda.manager.module;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.brilliance.mda.runtime.mda.util.*;
import com.fasterxml.jackson.annotation.*;
import java.util.regex.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.bo.Ljs;
import com.ceb.gjjs.mda.global.Platform;
import com.ceb.gjjs.mda.bo.Lje;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
* 
*/
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Ldtmod extends AbstractModule{
		private static Logger log = LoggerFactory.getLogger(Ldtmod.class);

        //报文保存根路径（配置文件读取）
        private String rootpath;

        //报文保存相对路径
        private String datapath;

        //保存文件完整目录
        private String fulldatapath;

        //文件数据流（类似格式转换使用）
        private IStream filstm = new StreamImpl();

        //文件存储流
        private IStream is = new StreamImpl();

        //翻译电报码
        private IControl tnsbtn;

        //返回
        private IControl rtnbtn;

        //原文
        private IControl orgmsg;

        //展开
        private IControl expmsg;

        //历史记录
        private IControl hisbtn;

        //刷新
        private IControl refres;

        //汇款报文展示
        private String msgdtl;

        //保存报文原文
        private String msg;

        //保存展开后报文
        private String emsg;


        @JsonIgnore
        @BDGetter
        private transient IPanel hispnl;

        @JsonIgnore
        @BDGetter
        private transient IPanel hislst;

        @JsonIgnore
        @BDGetter
        private transient IPanel hisbtnp;


	    @BDGetter
		private Ljs ljs;

		@BDGetter
		private ModuleList<Lje> ljelst;


		public Ldtmod()
        {
            super();
            setArgument();
        }

        public Ldtmod(String name,IModule parent)
        {
            super(name,parent);
            setArgument();
        }

        public void setArgument()
        {
        }

        /**
        *   source:ldtmod.@0002.script
        */
        @Rule(target="orgmsg",order=100)
        public boolean ruleOrgmsgN100(){
            IContext ctx = MdaEnv.getContext();
            
			if(!MdaUtils.isEmpty(this.getMsg()))
			{
			  this.setMsgdtl((String)this.getMsg());
			  
			}

			return true;
        }

        /**
        *   source:ldtmod.@0003.script
        */
        @Rule(target="expmsg",order=100)
        public boolean ruleExpmsgN100(){
            IContext ctx = MdaEnv.getContext();
            
			this.setEmsg(NULLSTR);
			if(MdaUtils.isEmpty(this.getEmsg()))
			{
			  if(!MdaUtils.isEmpty(this.getMsg()))
			  {
			    String em = expansion((String)this.getMsg(),"103",false);
			    if(em != null)
			    {
			      this.setMsgdtl(em);
			      this.setEmsg(em);
			    }

			  }
			}
			// else
			//   $msgdtl.setValue((String)$emsg.getValue());
			

			return true;
        }

        /**
        *   source:ldtmod.@0008.script
        */
        @Rule(target="tnsbtn",order=100)
        public boolean ruleTnsbtnN100(){
            IContext ctx = MdaEnv.getContext();
            
			String code = "";
			if(!MdaUtils.isEmpty(this.getMsg()))
			{
			  code = transCode(this.getMsg());
			}
			this.setMsgdtl( code);
			

			return true;
        }




        /**
        *   source:ldtmod.@0000.script
        */
        public String getStoreFileFullPathName(String filePath,String fileName, String fileType)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			if(filePath==null || fileName==null || fileType==null) return null;
			else
			{
			  return MdaUtils.catPath(filePath,"/"+fileName+"." + fileType);
			}
		}
        /**
        *   source:ldtmod.@0001.script
        */
        public String transfromStreamToString(IStream is)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			StringBuffer sb = new StringBuffer();
			try
			{
			  java.io.InputStream ins = is.getInputStream();
			  byte[] buf = new byte[1024];
			  int len = -1;
			  while ((len = ins.read(buf)) != -1)
			  {
			    String str = new String(buf,0,len);
			    sb.append(str);
			  }
			}
			catch(Exception e)
			{
			  log.debug("{}","Stream提取文本异常：" + e);
			}
			return sb.toString();
		}
        /**
        *   source:ldtmod.@0004.script
        */
        public String expansion(String arg,String msgTyp,boolean isFile)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  arg:传入参数，文件路径或直接为报文
			  msgTyp:报文类型
			  isFile:是否为文件，如是则为true，否则为false
			  */
			Object em = null;
			List tagLst = getTagLst(msgTyp);
			if(tagLst == null || tagLst.size()==0)
			  return null;
			String funNam = null;
			if(isFile)
			  funNam = "expansionViaFile";
			else
			  funNam = "expansionViaMsg";
			em = MdaUtils.invoke("com.brilliance.swift.MessageReader",funNam,arg,tagLst);
			if(em != null)
			  return em.toString();
			else
			  return null;
		}
        /**
        *   source:ldtmod.@0005.script
        */
        public List getTagLst(String msgtyp)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			if(msgtyp == null || "".equals(msgtyp))
			  return null;
			List tagList = new ArrayList();
			if (!msgtyp.startsWith("MT"))
			  msgtyp = "MT " + msgtyp;
			/*if (msgtyp.charAt(4) == '9')
			  msgtyp = "MT n" + msgtyp.substring(4);*/
			String sql = "SELECT inr,tag,fldnam,sta,cyc,mty,tno,midnam,msgdsp FROM mfd WHERE mty=? ORDER BY tno ASC";
			Argument<String> inr = new Argument<String>("inr");
			Argument<String> tag = new Argument<String>("tag");
			Argument<String> fldnam = new Argument<String>("fldnam");
			Argument<String> sta = new Argument<String>("sta");
			Argument<String> cyc = new Argument<String>("cyc");
			Argument<String> mty = new Argument<String>("mty");
			Argument<Integer> tno = new Argument<Integer>("tno");
			Argument<String> midnam = new Argument<String>("midnam");
			Argument<String> msgdsp = new Argument<String>("msgdsp");
			Platform.mdbExecuteSQL(sql,msgtyp);
			while(true)
			{
			  Platform.mdbFetchFields(tag,fldnam,sta,cyc,mty,tno,midnam,msgdsp);
			  if(ctx.getErrorCode() == NO_ERROR)
			  {
			    Map<String,String> tagMap = new HashMap<String,String>();
			    tagMap.put("inr", inr.value);
			    tagMap.put("tag", tag.value);
			    tagMap.put("fldnam",fldnam.value);
			    tagMap.put("sta", sta.value);
			    tagMap.put("cyc", cyc.value);
			    tagMap.put("mty", mty.value);
			    tagMap.put("tno", tno.value.toString());
			    tagMap.put("midnam", midnam.value);
			    tagMap.put("msgdsp",msgdsp.value);
			    tagList.add(tagMap);
			  }
			  else
			    break;
			}
			Platform.mdbCloseCursor();
			return tagList;
		}
        /**
        *   source:ldtmod.@0007.script
        */
        public String transCode(String msg)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			String tmsg = (String)MdaUtils.invoke("com.brilliance.swift.MessageReader","transMsg",msg);
			return tmsg;
		}































}