package com.ceb.gjjs.mda.manager.module;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.brilliance.mda.runtime.mda.util.*;
import com.fasterxml.jackson.annotation.*;
import java.util.regex.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.global.Platform;
import com.ceb.gjjs.mda.bo.Smt;
import com.ceb.gjjs.mda.bo.Doe;
import com.ceb.gjjs.mda.bo.Dot;
import com.ceb.gjjs.mda.bo.Msmlst;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
* 
*/
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Smtsub extends AbstractModule{
		private static Logger log = LoggerFactory.getLogger(Smtsub.class);

        //校验对账单结果信息
        private String retcode;



	    @BDGetter
		private Smt smt;

	    @BDGetter
		private Smt pre;

		@BDGetter
		private ModuleList<Smt> lst;

	    @BDGetter
		private Smt bof;

	    @BDGetter
		private Dot dot;

	    @BDGetter
		private Doe doe;

		@BDGetter
		private ModuleList<Msmlst> msmlst;


		public Smtsub()
        {
            super();
            setArgument();
        }

        public Smtsub(String name,IModule parent)
        {
            super(name,parent);
            setArgument();
        }

        public void setArgument()
        {
        }




        /**
        *   source:smtsub.@0000.script
        */
        public boolean checkAndSetStatement()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);

			/*
			  * 校验对账单信息
			  * @return  boolean 通过返回true 不通过返回false
			  */
			this.setRetcode(NULLSTR);
			if(MdaUtils.isEmpty(this.getSmt()))
			{
			  log.error("{}","待校验对账单信息为空,需要先查询对账单。");
			  return false;
			}
			String bic = this.getSmt().getBic();
			String smtameno = this.getSmt().getSmtameno();
			/// 首先根据账户行 、单类型、单号、币种、帐号  来判断是原有对账单还是新对账单
			MdaUtils.clear(this.getPre());
			Argument arg = new Argument("bic",bic).and("typ",this.getSmt().getTyp())
			  .and("smtameno",smtameno).and("cur",this.getSmt().getCur()).and("acc",this.getSmt().getAcc());
			//.and("yea",$smt\\yea);
			/// 20170508增加查询条件，仅查询30天内的对账单信息(大于当前日期-30)
			Date valdat = this.getSmt().getValdat();
			String tim = MdaUtils.format(MdaUtils.add(valdat,-30),"yyyy-MM-dd");
			log.debug("{}","check-in-30-days:"+tim);
			//valdat = Platform.parseDate(tim,"yyyy-MM-dd");
			//valdat = Platform.add(valdat,-30);
			arg = arg.and("valdat",GE,java.sql.Date.valueOf(tim));

			//增加实体查询条件
			arg= arg.and(ctx.absGet(Sysmod.class).getEtyArg(this.getSmt()));

			Platform.mdbRead(this.getPre(),arg);
			log.debug("{}","查询当前对账单 smt ="+ this.getPre().getInr()+" errtyp="+this.getPre().getErrtyp());
			/// 如果没有原单记录
			if(MdaUtils.isEmpty(this.getPre().getInr()))
			{
			  this.getSmt().setAccpty( ctx.absGet(Sysmod.class).getPtyinrForBIC(bic,this.getSmt()));
			  this.getSmt().setRecdat(MdaUtils.today());
			  this.getSmt().setSta( "0");
			  this.getSmt().setErrtyp( "0");
			}
			else
			{
			  /// 读取原对账单校验状态
			  /// 如果原对账单正确 当前页不做处理
			  if("0".equals(this.getPre().getSta()))
			  {
			    this.setRetcode("4");
			    log.error("{}","同编号对账单是正确对账单不做处理");
			    return false;
			  }
			  if(!MdaUtils.isEmpty(this.getSmt().getBegamt())
			    &&this.getSmt().getBegamt().compareTo(new BigDecimal(0))!=0)
			  {
			    this.getPre().setBegamt(this.getSmt().getBegamt());
			  }
			  if(!MdaUtils.isEmpty(this.getSmt().getEndamt())
			    &&this.getSmt().getEndamt().compareTo(new BigDecimal(0))!=0)
			  {
			    this.getPre().setEndamt(this.getSmt().getEndamt());
			  }
			  if(!MdaUtils.isEmpty(this.getSmt().getEndvalamt())
			    &&this.getSmt().getEndvalamt().compareTo(new BigDecimal(0))!=0)
			  {
			    this.getPre().setEndvalamt(this.getSmt().getEndvalamt());
			  }
			  this.getSmt().copyValue(this.getPre());
			}
			/// 校验上一对账单
			/// 如果当前对账单编号为1表示是今年第一个对账单不校验前对账单
			if("00001".equals(Platform.sfixPreToken(smtameno,"0",5)))
			{
			  log.debug("{}","smtno is "+smtameno);
			  return true;
			}
			String typ = this.getSmt().getTyp();
			/// 根据账户行对账单类型查询上一正确对账单
			Argument<String> pno = new Argument<String>("pno");
			String presql = "SELECT MAX(smtameno) pno FROM smt WHERE bic='"+bic+"' "
			  +" AND typ='"+typ+"' AND smtameno<'"+smtameno+"' "
			  +" AND acc ='"+this.getSmt().getAcc()+"' AND cur ='"+this.getSmt().getCur()+"' AND sta='0'";
			//+"' AND yea ='"+$smt\\yea+"'"
			// 20170508增加查询条件，仅查询30天内的对账单信息
			presql = presql + " AND to_char(valdat,'yyyy-MM-dd') >= '"+tim+"' ";
			//增加实体查询条件
			presql = presql + ctx.absGet(Sysmod.class).getEtySql(this.getSmt());

			Platform.mdbExecuteSQL(presql);
			Platform.mdbFetchFields(pno);
			Platform.mdbCloseCursor();
			String preno = getSmtnum(smtameno,-1);
			/// 上一单号不存在或与计算不符合 视为对账单号不连续
			if(MdaUtils.isEmpty(pno)||pno.value==null||!preno.equals(pno.value))
			{
			  this.getSmt().setSta("1");//问题
			  Platform.addError(MdaDriver.getDatafield(this.getSmt(),"errtyp"),"C");// 对账单号不连续
			  return false;
			}
			else
			{
			  Platform.removeError(MdaDriver.getDatafield(this.getSmt(),"errtyp"),"C");
			}
			preno = pno.value;
			MdaUtils.clear(this.getPre());
			/// 20170508增加查询条件，仅查询30天内的对账单信息
			Platform.mdbRead(this.getPre(),new Argument("smtameno",preno).and("bic",bic).and("typ",typ)
			  .and("acc",this.getSmt().getAcc()).and("cur",this.getSmt().getCur())
			  .and("sta","0").and(ctx.absGet(Sysmod.class).getEtyArg(this.getSmt())).and("valdat",GE,java.sql.Date.valueOf(tim)));
			//.and("yea",$smt\\yea)
			log.debug("{}","查询上个对账单 smt ="+ this.getPre().getInr()+" errtyp="+this.getPre().getErrtyp());
			BigDecimal preendamt = this.getPre().getEndamt();
			BigDecimal curbegamt = this.getSmt().getBegamt();
			if(preendamt.compareTo(curbegamt)!=0)
			{
			  this.getSmt().setSta("1");//问题
			  Platform.addError(MdaDriver.getDatafield(this.getSmt(),"errtyp"),"8");// 对账单金额不连续
			  return false;
			}
			else
			{
			  Platform.removeError(MdaDriver.getDatafield(this.getSmt(),"errtyp"),"8");
			}
			log.info("{}","SMT ERR TYP................................:"+this.getSmt().getErrtyp());
			log.debug("{}","check SMT over");
			return true;
		}
        /**
        *   source:smtsub.@0001.script
        */
        public void createDotRecsFormSmtinr(String smtinr)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			if(MdaUtils.isEmpty(smtinr))
			  return;
			/*
			  String sql="SELECT t.accpty,t.bic,d.inr,d.cur,d.amt,d.valdat,d.dcflg,d.ownref,d.othref,t.recdat,t.smtno,t.ref,t.typ,d.busflg,d.inf,p.pagnum,t.acc,d.accdat FROM smd d,smt t,smp p "
			  +"WHERE t.inr='"+smtinr+"' AND d.smtinr=t.inr AND d.smpinr=p.inr AND p.smtinr=t.inr AND p.sta!='3' ";
			  */
			String sql="SELECT t.accpty,t.bic,d.inr,d.cur,d.amt,d.valdat,d.dcflg,d.ownref,d.othref,i.gendat recdat,t.recdat,t.smtno,t.ref,t.typ,d.busflg,d.inf,p.pagnum,t.acc,d.accdat FROM smd d,smt t,smp p,mpi i "
			  +"WHERE t.inr='"+smtinr+"' AND d.smtinr=t.inr AND d.smpinr=p.inr AND p.smtinr=t.inr AND p.msdinr=i.inr AND p.sta!='3' ";
			Argument<String> accpty = new Argument<String>("accpty");
			Argument<String> bic = new Argument<String>("bic");
			Argument<String> ainr = new Argument<String>("inr");
			Argument<String> acur = new Argument<String>("cur");
			Argument<BigDecimal> aamt = new Argument<BigDecimal>("amt");
			Argument<Date> avaldat = new Argument<Date>("valdat");
			Argument<String> adcflg = new Argument<String>("dcflg");
			Argument<String> aownref = new Argument<String>("ownref");
			Argument<String> aothref = new Argument<String>("othref");
			Argument<Date> recdat = new  Argument<Date>("recdat");//Alex 改过这里
			Argument<String> smtno = new Argument<String>("smtno");
			Argument<String> smtref = new Argument<String>("ref");
			Argument<String> busflg = new Argument<String>("busflg");
			Argument<String> inf = new Argument<String>("inf");
			Argument<String> pagnum = new Argument<String>("pagnum");
			Argument<String> mty=new Argument<String>("typ");
			Argument<String> acc = new Argument<String>("acc");
			Argument<Date> accdat = new Argument<Date>("accdat");
			Platform.mdbExecuteSQL(sql);
			List<Object[]> list =new ArrayList<Object[]>();
			while(true)
			{
			  Platform.mdbFetchFields(accpty,bic,ainr,acur,aamt,avaldat,adcflg,aownref,aothref,recdat,smtno,smtref,busflg,inf,pagnum,mty,acc,accdat);
			  int errorCode = ctx.getErrorCode();
			  if(errorCode==NO_ERROR)
			  {
			    Object[] objs = new Object[]{accpty.value,bic.value,ainr.value,acur.value,aamt.value,avaldat.value,adcflg.value,aownref.value,aothref.value,recdat.value,smtno.value,smtref.value,busflg.value,inf.value,pagnum.value,mty.value,acc.value,accdat.value};
			    list.add(objs);
			  }
			  else
			    break;
			}
			log.debug("{}","smt"+smtinr+"smd size"+list.size());
			Platform.mdbCloseCursor();
			// 生成对账单明细待对账记录、、
			// 读取对账单明细列表
			for(int i=0;i<list.size();i++)
			{
			  Object[] objs = list.get(i);
			  MdaUtils.clear(this.getDot());
			  this.getDot().setObjtyp("smd");
			  this.getDot().setAccpty(objs[0].toString());
			  this.getDot().setActbic(objs[1].toString());
			  this.getDot().setCur(objs[3].toString());
			  this.getDot().setAmt((BigDecimal)objs[4]);
			  this.getDot().setValdat((Date)objs[5]);
			  this.getDot().setDcflg(objs[6].toString());
			  this.getDot().setOwnref(objs[7].toString());
			  if(objs[8]!=null)
			    this.getDot().setOthref(objs[8].toString());
			  this.getDot().setSta("1");
			  this.getDot().setOffsta("2");
			  this.getDot().setCredat(MdaUtils.today());
			  this.getDot().setObjinr(objs[2].toString());
			  this.getDot().setRsptim((Date)objs[9]);
			  this.getDot().setStanum((String)objs[10]);
			  this.getDot().setStaref((String)objs[11]);
			  this.getDot().setBusflg((String)objs[12]);
			  this.getDot().setInf((String)objs[13]);
			  this.getDot().setSeqnum((String)objs[14]);
			  this.getDot().setMty((String)objs[15]);
			  String accc=(String)objs[16];
			  log.info("{}","(String)objs[16]"+accc);
			  this.getDot().setAcc(accc);
			  this.getDot().setEntdat((Date)objs[17]);
			  String dotinr = Platform.sdbCounter("dot");
			  this.getDot().setInr(dotinr);
			  this.getDot().setLstdat(new Date());
			  //log.debug("new dotinr: " + dotinr+" objinr"+$dot\\objinr);
			  this.getDot().setIsdone("1");
			  this.getDot().setRecsta("2");

			  //关联所属实体
			  ctx.absGet(Sysmod.class).setEtyAndPty(this.getDot(),this.getSmt());

			  Platform.mdbInsert(this.getDot());
			  this.getDoe().setDotinr( this.getDot().getInr());
			  this.getDoe().setFrmnam( ctx.getTransName());
			  this.getDoe().setUsr( MdaEnv.getLoginUser());
			  this.getDoe().setOpdate(new Date());
			  this.getDoe().setBak( "自动生成待对账记录");
			  Platform.sdbInsert(this.getDoe());
			}
		}
        /**
        *   source:smtsub.@0003.script
        */
        public String getErrorMessageFroType(String errtyp)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			if(MdaUtils.isEmpty(errtyp)||"0".equals(errtyp))
			  return "正常";
			String errmsgStr = "";
			int len = errtyp.length();
			for(int i =0;i< len;i++)
			{
			  String errlbl = MdaUtils.getCodetableLabel("errtyp",String.valueOf(errtyp.charAt(i)));
			  errmsgStr = errmsgStr + errlbl+"\r\n";
			}
			errmsgStr = errmsgStr.substring(0,errmsgStr.length() - 2);
			return errmsgStr;
		}
        /**
        *   source:smtsub.@0004.script
        */
        public void updateNextSmts()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 如果当前对账单状态正确
			  * 顺次更新更新下一对账单状态，直到没有对账单或对账单不是正常对账单为止
			  */
			if(MdaUtils.isEmpty(this.getSmt())||MdaUtils.isEmpty(this.getSmt().getInr()))
			  return;
			// 正确对账单才更新下一对账单状态
			if(MdaUtils.isEmpty(this.getSmt().getSta())||!"0".equals(this.getSmt().getSta()))
			  return;
			int step =1;
			boolean goon = true;
			BigDecimal endamt = this.getSmt().getEndamt();
			while(goon)
			{
			  /// 查找下个对账单 去掉单不连续状态 C
			  MdaUtils.clear(this.getPre());
			  ///20170509修改
			  Date valdat = this.getSmt().getValdat();
			  String tim = MdaUtils.format(MdaUtils.add(valdat,-30),"yyyy-MM-dd");
			  log.debug("{}","check-before-30-days:"+tim);
			  Platform.mdbRead(this.getPre(),new Argument("bic",this.getSmt().getBic()).and("typ",this.getSmt().getTyp())
			    .and("smtameno",getSmtnum(this.getSmt().getSmtameno(),step))
			    .and("cur",this.getSmt().getCur()).and("acc",this.getSmt().getAcc())
			    .and("valdat",GE,java.sql.Date.valueOf(tim)).and(ctx.absGet(Sysmod.class).getEtyArg(this.getSmt())));//20170509新加
			  //.and("yea",$smt\\yea));
			
			  log.debug("{}","查询下个对账单  ="+ this.getPre().getInr()+" 单号 ="+this.getPre().getSmtameno()+" errtyp ="+this.getPre().getErrtyp());
			  if(MdaUtils.isEmpty(this.getPre())||MdaUtils.isEmpty(this.getPre().getInr()))
			    return;
			  ///去除后一对账单不连续状态
			  Platform.removeError(MdaDriver.getDatafield(this.getPre(),"errtyp"),"C");
			  /// 校验后一对账单金额连续状态
			  BigDecimal begamt = this.getPre().getBegamt();
			  log.trace("{}","end "+endamt+" to "+begamt);
			  if(endamt.compareTo(begamt)!=0)
			  {
			    this.getPre().setSta("1");//问题
			    Platform.addError(MdaDriver.getDatafield(this.getPre(),"errtyp"),"8");// 对账单金额不连续
			    log.error("{}","对账单 "+this.getPre().getInr()+"金额不连续");
			  }
			  else
			  {
			    Platform.removeError(MdaDriver.getDatafield(this.getPre(),"errtyp"),"8");
			  }
			  /// 每次比较完后重置 期末金额
			  endamt = this.getPre().getEndamt();
			  /// 如果下个对账单没有其他问题了置正常状态并生成明细待对账记录
			  if("".equals(this.getPre().getErrtyp()))
			  {
			    this.getPre().setSta( "0");
			    this.getPre().setErrtyp( "0");
			    createDotRecsFormSmtinr(this.getPre().getInr());
			    step++;
			  }
			  else
			    goon = false;
			  /// 更新下个对账单状态
			  Platform.mdbUpdate(this.getPre());
			}
		}
        /**
        *   source:smtsub.@0005.script
        */
        public String getSmtnum(String smtnum,int step)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 根据传入的页码和加减值，返回长度为5的页码字符串
			  * - String pagnum 传入的页码
			  * - int step 页码加减值
			  * @ 格式化以后的新页码
			  */
			if(MdaUtils.isEmpty(smtnum))
			  smtnum="00000";
			if(step==0)
			  return smtnum;
			int curno = Integer.valueOf(smtnum);
			curno = curno +step ;
			String newsmtnum = String.valueOf(curno);
			newsmtnum = Platform.sfixPreToken(newsmtnum,"0",5);
			return newsmtnum;
		}
        /**
        *   source:smtsub.@0006.script
        */
        public int checkStatementBaseInfo(String bic,String cur,String act)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 校验对账单报文中的bic 帐号 币种是否正确
			  * @return int  0 正常  1账户行不存在 2币种不存在 3账户不存在
			  */
			Platform.setDefaultDB("swd");
			if(MdaUtils.isEmpty(bic))
			  return 1;//账户行不存在
			if(!ctx.absGet(Sysmod.class).isHodBank(bic))
			{
			  log.debug("{}","账户行 "+bic+" 非我行账户行");
			  return 1;
			}
			/// 格式化 mpi表中的帐号
			String formatAct = Platform.format4Equals(act,false);
			log.trace("{}","act = "+act);
			log.trace("{}","formatAct = "+formatAct);
			String sql="SELECT a.cur,a.formid FROM pty p,act a WHERE ptytyp "
			  +"LIKE '%T%' AND EXTKEY='"+bic+"' AND p.inr=a.actbnkinr ";

			//增加实体查询条件
			sql = sql + ctx.absGet(Sysmod.class).getEtySql("p",this.getSmt());

			sql = sql + " GROUP BY a.cur,a.formid ";
			Argument<String> curA = new Argument<String>("cur");
			Argument<String> actA = new Argument<String>("formid");
			Platform.mdbExecuteSQL(sql);
			Map<String,String> map = new HashMap<String,String>();
			while(true)
			{
			  Platform.mdbFetchFields(curA,actA);
			  int errorCode = ctx.getErrorCode();
			  if(errorCode==NO_ERROR)
			  {
			    map.put(actA.value,curA.value);
			  }
			  else
			    break;
			}
			Platform.mdbCloseCursor();
			log.debug("{}","bic info size "+map.size());
			for(String key : map.keySet())
			{
			  log.debug("{}","act|"+key+"|cur|"+map.get(key)+"|");
			}
			if(!map.containsKey(formatAct))
			{
			  log.debug("{}","账户行 "+bic+" 中不存在账户 "+act);
			  return 2;
			}
			if(!map.containsValue(cur))
			{
			  log.debug("{}","账户行 "+bic+" 中不存在币种 "+cur);
			  return 3;
			}
			return 0;
		}
        /**
        *   source:smtsub.@0007.script
        */
        public String setSmtStatusForAllPage()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  *	读取对账单下属页的错误类型加到对账单错误类型并返回
			  * @ return String 在原有状态基础上返回计算后的单错误状态
			  */
			if(MdaUtils.isEmpty(this.getSmt())||MdaUtils.isEmpty(this.getSmt().getInr()))
			  return null;
			String smterrtyp = this.getSmt().getErrtyp();
			if(MdaUtils.isEmpty(smterrtyp))
			  smterrtyp="";
			log.debug("{}","smterrtyp begin"+smterrtyp);
			/// 如果是7重复单 不更新页状态
			if("7".equals(smterrtyp))
			  return smterrtyp;
			// 先从对账单错误中去除1-7的页错误状态和0正常状态,
			smterrtyp = smterrtyp.replaceAll("[0-7]", "");
			String smtinr = this.getSmt().getInr();
			/// 查询数据库校验对账单下是否有错误的页信息
			String sql = "SELECT errtyp FROM smp WHERE  smtinr='"+smtinr+"' AND sta='1'  ";
			Argument<String> errtyp = new Argument<String>("errtyp");
			String smperrtyp = "";// 页错误状态汇总字符串
			List<String> typlst = new ArrayList<String>();
			Platform.mdbExecuteSQL(sql);
			while(true)
			{
			  Platform.mdbFetchFields(errtyp);
			  int err = ctx.getErrorCode();
			  if(err==NO_ERROR)
			  {
			    if(!errtyp.value.contains("7"))// 不计注销页的状态
			    smperrtyp = smperrtyp+errtyp.value;
			  }
			  else break;
			}
			Platform.mdbCloseCursor();
			smperrtyp = smperrtyp.replaceAll("0", "");
			smperrtyp = smperrtyp.replaceAll("null", "");
			smperrtyp = smperrtyp.replaceAll("NULL", "");
			log.debug("{}","smperrtyp "+smperrtyp);
			for(int i=1;i<=6;i++)// 只循环到6 页注销状态不更新到单
			{
			  if(smperrtyp.indexOf(String.valueOf(i))>=0)
			    smterrtyp = smterrtyp+String.valueOf(i);
			}
			this.getSmt().setErrtyp( smterrtyp);
			log.debug("{}","end smterrtyp"+this.getSmt().getErrtyp());
			return smterrtyp;
		}
        /**
        *   source:smtsub.@0013.script
        */
        public String setSmtStatusForSmpMap(Map<String,String> map)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  *	读取对账单下属页的错误类型加到对账单错误类型并返回
			  * @ return String 在原有状态基础上返回计算后的单错误状态
			  */
			if(MdaUtils.isEmpty(this.getSmt())||MdaUtils.isEmpty(this.getSmt().getInr()))
			  return null;
			String smterrtyp = this.getSmt().getErrtyp();
			if(MdaUtils.isEmpty(smterrtyp))
			  smterrtyp="";
			log.debug("{}","smterrtyp begin"+smterrtyp);
			/// 如果是7重复单 不更新页状态
			if("7".equals(smterrtyp))
			  return smterrtyp;
			// 先从对账单错误中去除1-7的页错误状态和0正常状态,
			smterrtyp = smterrtyp.replaceAll("[0-7]", "");
			String smtinr = this.getSmt().getInr();
			/// 查询数据库校验对账单下是否有错误的页信息
			String smperrtyp = "";// 页错误状态汇总字符串
			for(String key : map.keySet())
			{
			  log.debug("{}"," inr "+key+" errtyp "+map.get(key));
			  if(!map.get(key).contains("7"))// 不计注销页的状态
			  smperrtyp = smperrtyp+map.get(key);
			}
			smperrtyp = smperrtyp.replaceAll("0", "");
			smperrtyp = smperrtyp.replaceAll("null", "");
			smperrtyp = smperrtyp.replaceAll("NULL", "");
			log.debug("{}","smperrtyp "+smperrtyp);
			for(int i=1;i<=6;i++)// 只循环到6 页注销状态不更新到单
			{
			  if(smperrtyp.indexOf(String.valueOf(i))>=0)
			    smterrtyp = smterrtyp+String.valueOf(i);
			}
			this.getSmt().setErrtyp( smterrtyp);
			log.debug("{}","end smterrtyp"+this.getSmt().getErrtyp());
			return smterrtyp;
		}










}