package com.ceb.gjjs.mda.manager.module;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.brilliance.mda.runtime.mda.util.*;
import com.fasterxml.jackson.annotation.*;
import java.util.regex.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.global.Platform;
import com.ceb.gjjs.mda.bo.Mpi;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
* 
*/
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Smtsvr extends AbstractModule{
		private static Logger log = LoggerFactory.getLogger(Smtsvr.class);

        //对账单类型
        private String typ;

        //账户行bic
        private String bic;

        //是否生成新的对账单
        private String newsmt;

        //61明细
        private Object tag61;

        //报文创建交易
        private String smhcrefrm;



	    @BDGetter
		private Smtsub smtsub;

	    @BDGetter
		private Smpsub smpsub;

	    @BDGetter
		private Smdsub smdsub;

		@Module(getter = false, setter = false)
		private transient ModuleHolder<Mpi> mpi;


		public Smtsvr()
        {
            super();
            setArgument();
        }

        public Smtsvr(String name,IModule parent)
        {
            super(name,parent);
            setArgument();
        }

        public void setArgument()
        {
            this.getSmpsub().setSmtHolder(()->this.getSmtsub().getSmt());
            this.getSmdsub().setSmtHolder(()->this.getSmtsub().getSmt());
        }




        /**
        *   source:smtsvr.@0000.script
        */
        public boolean doHandle(String path)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 对账单解析子模块处理方法。用于解析940或950对账单
			  - path:String 报文文件路径
			  */
			log.info("{}","smtsvr dohandle");
			this.setTyp(NULLSTR);
			this.setBic(NULLSTR);
			this.setNewsmt(NULLSTR);

			//关联所属实体
			this.getSmtsub().getSmt().setTyp( "950");
			ctx.absGet(Sysmod.class).setEtyAndPty(this.getSmtsub().getSmt(),this.getMpi());
			log.info("{}","smt-etyinr1:"+this.getSmtsub().getSmt().getEtyinr());

			/// 对账单基本信息校验
			/// 校验是否为我行账户行
			this.setTyp(this.getMpi().getSubtyp());
			this.setBic( this.getMpi().getSndbic());
			log.debug("{}","typ="+this.getTyp());
			log.debug("{}","bic="+this.getBic());
			/// 校验对账单报文基本信息是否正确  0 正常  1账户行不存在 2币种不存在 3账户不存在
			int retcode = this.getSmtsub().checkStatementBaseInfo(this.getBic(),this.getMpi().getCur(),this.getMpi().getAct());
			log.debug("{}","ret code1:"+retcode);
			switch (retcode)
			{
			  case 1:
			  this.getMpi().setExcmsg( "账户行 "+this.getBic()+" 非我行账户行");
			  return false;
			  case 2:
			  this.getMpi().setExcmsg( "账户行 "+this.getBic()+" 中不存在账户 "+this.getMpi().getAct());
			  return false;
			  case 3:
			  this.getMpi().setExcmsg( "账户行 "+this.getBic()+" 中不存在币种 "+this.getMpi().getCur());
			  return false;
			}
			/// 解析对账单报文获取数据分别保存在smt 和smp
			/// 如果解析失败则不再进行后续逻辑并返回false
			retcode = loadStatementData(path);
			log.debug("{}","ret code2:"+retcode);
			switch (retcode)
			{
			  case 4:
			  this.getMpi().setExcmsg( "报文信息不存在");
			  return false;
			  case 5:
			  this.getMpi().setExcmsg( "对账单中币种不一致");
			  return false;
			}

			//关联所属实体
			ctx.absGet(Sysmod.class).setEtyAndPty(this.getSmtsub().getSmt(),this.getMpi());
			log.info("{}","smt-etyinr2:"+this.getSmtsub().getSmt().getEtyinr());
			/// 先校验对账单信息
			if(!this.getSmtsub().checkAndSetStatement())
			  log.error("{}","对账单校验不通过");
			/// 如果对账单已接收完毕则当前报文不处理
			if("4".equals(this.getSmtsub().getRetcode()))
			{
			  this.getMpi().setExcmsg( "对账单 "+this.getSmtsub().getSmt().getSmtameno()+"已是正确对账单，不再处理");
			  return false;
			}
			if(MdaUtils.isEmpty(this.getSmtsub().getSmt().getInr()))
			{
			  this.setNewsmt("true");
			  Platform.sdbCounter(this.getSmtsub().getSmt());
			}
			this.getSmpsub().getSmp().setSmtinr( this.getSmtsub().getSmt().getInr());
			/// 校验对账单页信息
			if(!this.getSmpsub().checkAndSetPage())
			  log.error("{}","对账单页校验不通过");
			/// 保存明细信息
			saveDetail();
			/// 更新页状态
			String smperrtyp = this.getSmpsub().getSmp().getErrtyp();
			if(smperrtyp==null||"".equals(smperrtyp.replaceAll("0", "")))
			{
			  this.getSmpsub().getSmp().setErrtyp( "0");
			  this.getSmpsub().getSmp().setSta( "0");
			}
			else
			{
			  this.getSmpsub().getSmp().setSta( "1");
			}

			/// 获取所有页的状态
			Map<String,String> map = (Map<String,String>)this.getSmpsub().getErrmap();
			/// 计算并更新单状态
			this.getSmtsub().setSmtStatusForSmpMap(map);
			Platform.addError(MdaDriver.getDatafield(this.getSmtsub().getSmt(),"errtyp"),this.getSmpsub().getSmp().getErrtyp());
			Platform.addError(MdaDriver.getDatafield(this.getSmtsub().getSmt(),"errtyp"),this.getSmpsub().getErrtyp());
			String smterrtyp = this.getSmtsub().getSmt().getErrtyp();
			smterrtyp = smterrtyp.replaceAll("0", "");
			if("".equals(smterrtyp))
			{
			  this.getSmtsub().getSmt().setSta( "0");
			  this.getSmtsub().getSmt().setErrtyp( "0");
			  /// 生成当前对账单当前页的明细待对账信息
			  log.debug("{}","生成对账单当前页明细待对账信息 size="+this.getSmdsub().getLst().fullSize());
			  this.getSmdsub().createDotRecsFormSmdList(this.getSmtsub().getSmt().getAccpty(),this.getBic());
			  // 生成当前单已入库正确页明细的待对账信息
			  this.getSmtsub().createDotRecsFormSmtinr(this.getSmtsub().getSmt().getInr());
			  /// 顺次更新原对账单的状态
			  this.getSmtsub().updateNextSmts();
			}
			else
			{
			  this.getSmtsub().getSmt().setSta( "1");
			}
			log.debug("{}","smt inr "+this.getSmtsub().getSmt().getInr()+" sta "+this.getSmtsub().getSmt().getSta()+" errtyp "+this.getSmtsub().getSmt().getErrtyp());
			log.debug("{}","smp inr "+this.getSmpsub().getSmp().getInr()+" sta "+this.getSmpsub().getSmp().getSta()+" errtyp "+this.getSmpsub().getSmp().getErrtyp());
			/// 将校验过的对账单和页信息插入数据库
			if("true".equals(this.getNewsmt()))
			{
			  Platform.mdbInsert(this.getSmtsub().getSmt());
			  log.debug("{}","new smt");
			}
			else
			{
			  Platform.mdbUpdate(this.getSmtsub().getSmt());
			  log.debug("{}","update smt");
			}
			Platform.mdbInsert(this.getSmpsub().getSmp());
			//更新账户表余额，并判断是否生成划款指令
			updateActAndGenerOrder();
			/// 处理完毕无论是否是问题对账单都返回true
			return true;
		}
        /**
        *   source:smtsvr.@0001.script
        */
        public int loadStatementData(String path)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 读取对账单报文文件，把获得的信息保存在本模块下的变量中。
			  - path:String 报文文件路径
			  */
			/// 校验文件是否存在
			if(MdaUtils.isEmpty(path)||!Platform.mfileExists(path))
			{
			  return 4;
			}
			MdaUtils.clear(this.getSmtsub().getPre());
			MdaUtils.clear(this.getSmtsub().getSmt());
			MdaUtils.clear(this.getSmpsub().getPre());
			MdaUtils.clear(this.getSmpsub().getSmp());
			this.setTag61(null);
			/// 将解析的报文基本信息保存到 $tags中
			String clsname = "com.brilliance.swift.MessageReader";
			Object tagObj = MdaUtils.invoke(clsname,"simpleSplit",path);
			if(tagObj == null)
			{
			  log.error("{}","解析基本栏位出错 ");
			  this.getMpi().setExcmsg("解析报文基本栏位出错，请检查报文文件。");
			  //基本解析栏位出错   返回
			  return 4;
			}
			/// 将报文61域的明细  保存到 $tagdetails中
			List<String[]> tags = (List<String[]>) tagObj;
			List<String> tag61 = new ArrayList<String>();
			int taglen = tags.size();//读取对账单参考号
			log.debug("{}","taglen="+taglen);
			String bgmStr = null;
			String cur = null;
			String pagnum = "1";// 页号
			String smtno = "1";// 单号
			String act = null;
			boolean isbegin = false;
			boolean isend = false;
			BigDecimal all = new BigDecimal(0);
			// 先生成当前页的inr 用于保存明细
			log.info("{}","hello");
			String inr=Platform.sdbCounter(this.getSmpsub().getSmp());
			log.info("{}","smp inr is "+this.getSmpsub().getSmp().getInr());
			log.info("{}","return inr is "+inr);
			for(int i=0;i<taglen ;i++ )
			{
			  String[] tag = tags.get(i);
			  if("20".equals(tag[0]))
			  {
			    this.getSmtsub().getSmt().setRef(tag[1]);
			    this.getSmpsub().getSmp().setRef(tag[1]);
			  }
			  else if("25".equals(tag[0]))
			  {
			    act = tag[1];
			    act = act.replaceAll("\\s","");
			    this.getSmtsub().getSmt().setAcc(act);
			  }
			  else if("28C".equals(tag[0]))
			  {
			    int idx = tag[1].indexOf("/");
			    if(idx >-1)
			    {
			      smtno = tag[1].substring(0,idx);
			      if(!tag[1].endsWith("/"))
			        pagnum = tag[1].substring(idx+1);
			    }
			    else
			      smtno = tag[1];
			    // 单号和页号前面补0
			    smtno =Platform.sfixPreToken(smtno,"0",5);
			    pagnum =Platform.sfixPreToken(pagnum,"0",5);
			    log.info("{}","对账单号："+smtno);
			    log.info("{}","当前页号："+pagnum);
			    this.getSmpsub().getSmp().setPagnum(pagnum);
			    this.getSmpsub().getSmp().setPagamenum(pagnum);
			    this.getSmtsub().getSmt().setSmtno(smtno);
			    this.getSmtsub().getSmt().setSmtameno( smtno);
			    this.getSmpsub().getSmp().setSmtno(smtno);
			  }
			  else if(tag[0].matches("60[FM]"))
			  {
			    isbegin = tag[0].endsWith("F");
			    bgmStr = tag[1].substring(10).replace(',','.');
			    if(tag[1].startsWith("D"))
			      bgmStr = "-"+bgmStr;
			    BigDecimal  begamt = MdaUtils.parseDecimal(bgmStr+"0");
			    cur = tag[1].substring(7,10);
			    //获取期初金额
			    this.getSmpsub().getSmp().setBegamt(begamt);
			    this.getSmpsub().getSmp().setNewbegamt(begamt);
			    this.getSmpsub().getSmp().setCur(cur);
			    this.getSmtsub().getSmt().setCur(cur);
			    /// 出现币种错误则不再解析
			    if(!cur.equals(this.getMpi().getCur()))
			    {
			      return 5;
			    }
			  }
			  else if("61".equals(tag[0]))
			  {
			    tag61.add(tag[1]);
			  }
			  else if(tag[0].matches("62[FM]"))
			  {
			    isend = tag[0].endsWith("F");
			    bgmStr = tag[1].substring(10).replace(',','.');
			    if(tag[1].startsWith("D"))
			      bgmStr = "-"+bgmStr;
			    BigDecimal  endamt = MdaUtils.parseDecimal(bgmStr+"0");
			    //获取期初金额
			    this.getSmpsub().getSmp().setEndamt(endamt);
			    this.getSmpsub().getSmp().setNewendamt(endamt);
			    cur = tag[1].substring(7,10);
			    this.getSmtsub().getSmt().setCur(cur);
			    this.getSmpsub().getSmp().setCur(cur);
			    /// 出现币种错误则不再解析
			    if(!cur.equals(this.getMpi().getCur()))
			      return 5;
			    String valdat = tag[1].substring(1,7);
			    this.getSmtsub().getSmt().setValdat(MdaUtils.parseDate(valdat,"yyMMdd"));
			    this.getSmpsub().getSmp().setValdat(MdaUtils.parseDate(valdat,"yyMMdd"));
			  }
			  else if("64".equals(tag[0]))
			  {
			    bgmStr = tag[1].substring(10).replace(',','.');
			    if(tag[1].startsWith("D"))
			      bgmStr = "-"+bgmStr;
			    BigDecimal  endamt = MdaUtils.parseDecimal(bgmStr+"0");
			    log.debug("{}","set eva 64 " + endamt);
			    this.getSmtsub().getSmt().setEndvalamt(endamt);
			    this.getSmpsub().getSmp().setEndvalamt(endamt);
			  }
			}
			this.setTag61(tag61);
			this.getSmtsub().getSmt().setTyp( this.getTyp());
			this.getSmtsub().getSmt().setBic( this.getBic());
			this.getSmtsub().getSmt().setYea( MdaUtils.format(this.getSmtsub().getSmt().getValdat(),"yyyy"));
			this.getSmpsub().getSmp().setMsdinr( this.getMpi().getInr());
			/// 设定期初期末金额
			log.debug("{}","year "+this.getSmtsub().getSmt().getYea());
			log.debug("{}","isbegin "+isbegin);
			log.debug("{}","isend "+isend);
			/// 使用chk用做首末页标志
			String chk = this.getSmpsub().getSmp().getChk();
			if(chk==null)
			  chk ="";
			chk = chk.trim();
			if(isbegin)
			{
			  // 设定是首页
			  chk = chk+"O";
			  this.getSmtsub().getSmt().setBegamt(this.getSmpsub().getSmp().getNewbegamt());
			  log.trace("{}","smt begamt "+this.getSmtsub().getSmt().getBegamt());
			}
			if(isend)
			{
			  // 设定是末页
			  chk = chk+"X";
			  this.getSmtsub().getSmt().setEndamt(this.getSmpsub().getSmp().getNewendamt());
			  log.trace("{}","smt endamt "+this.getSmtsub().getSmt().getEndamt());
			  /// 如果没有64域 则期末可用余额设为期末余额
			  if(MdaUtils.isEmpty(this.getSmtsub().getSmt().getEndvalamt()))
			  {
			    this.getSmtsub().getSmt().setEndvalamt(this.getSmpsub().getSmp().getNewendamt());
			    this.getSmpsub().getSmp().setEndvalamt(this.getSmpsub().getSmp().getNewendamt());
			  }
			  log.trace("{}","smt endvalamt "+this.getSmtsub().getSmt().getEndvalamt());
			}
			this.getSmpsub().getSmp().setChk( chk);
			/// 设定业内试算金额
			this.getSmpsub().getSmp().setPagamt(all);
			return 0;
		}
        /**
        *   source:smtsvr.@0003.script
        */
        public void saveDetail()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 从$tagdetails获取已解析的对账单明细数据，校验并入库。
			  */
			MdaUtils.clear(this.getSmdsub().getLst());
			log.debug("{}","checkAndSaveDetail");
			if(MdaUtils.isEmpty(this.getTag61()))
			{
			  log.debug("{}","61域名系信息不存在");
			  return;
			}
			List<String> lst61 = (List<String>)this.getTag61();
			log.debug("{}","61 size="+lst61.size());
			BigDecimal all = new BigDecimal(0);
			for(int i=0;i<lst61.size();i++)
			{
			  MdaUtils.clear(this.getSmdsub().getSmd());
			  String line = lst61.get(i) ;
			  line = line.replaceAll("\n", "\r").replaceAll("\r\r", "\r");
			  //log.debug("line="+line);
			  int pos = 0;
			  String valdat = line.substring(0,6);
			  this.getSmdsub().getSmd().setValdat(MdaUtils.parseDate(valdat,"yyMMdd"));
			  //log.debug("1"+$smdsub\\smd\\valdat);
			  pos += 6;
			  String accdat = line.substring(pos,pos+4);
			  if(accdat.matches("\\d{4}"))
			  {
			    this.getSmdsub().getSmd().setAccdat(MdaUtils.parseDate(valdat.substring(0,2)+accdat,"yyMMdd"));
			    pos += 4;
			  }
			  else
			  {
			    this.getSmdsub().getSmd().setAccdat(MdaUtils.parseDate(valdat,"yyMMdd"));//如果没有填记账 日期，直接填为起息日
			  }
			  //log.debug("2"+$smdsub\\smd\\accdat);
			  String dcflg = line.substring(pos,pos+2);
			  if(dcflg.equals("RD") || dcflg.equals("RC"))
			  {
			    pos += 2;
			  }
			  else
			  {
			    dcflg = dcflg.substring(0,1);
			    pos += 1;
			  }
			  this.getSmdsub().getSmd().setDcflg(dcflg);
			  //log.debug("3"+dcflg);
			  //保存到了借贷标识
			  if(line.charAt(pos)>='A' && line.charAt(pos)<='Z')
			  {
			    this.getSmdsub().getSmd().setFuncod(line.substring(pos,pos+1));
			    pos++;
			  }
			  //log.debug("4"+$smdsub\\smd\\funcod);
			  //需找金额字串
			  int idx = line.indexOf(",",pos);
			  //log.debug("5idx"+idx+"|pos"+pos);
			  idx++;
			  while(true)
			  {
			    if(line.charAt(idx)>='0' && line.charAt(idx)<='9')
			      idx++;
			    else
			      break;
			  }
			  String amtStr = line.substring(pos,idx).replace(',','.')+"0";
			  //log.trace("5str"+amtStr);
			  BigDecimal amt = new BigDecimal(amtStr);
			  this.getSmdsub().getSmd().setAmt(amt);
			  //log.trace("5"+amt);
			  pos = idx;
			  //寻找业务类型
			  String bustyp = line.substring(pos,pos+4);
			  this.getSmdsub().getSmd().setBusflg(bustyp);
			  //log.debug("6"+bustyp);
			  pos += 4;
			  // 获取描述信息
			  int rn = line.indexOf("\r",pos);
			  if(rn>=pos)
			  {
			    this.getSmdsub().getSmd().setInf(line.substring(rn+1));
			    line = line.substring(0,rn);
			  }
			  //获取参考号
			  String tagl = "/"+"/";
			  int refIdx = line.indexOf(tagl,pos);
			  if(refIdx >= pos)
			  {
			    this.getSmdsub().getSmd().setOwnref(line.substring(pos,refIdx).trim());
			    pos = refIdx + 2;
			    this.getSmdsub().getSmd().setOthref(line.substring(pos).trim());
			  }
			  else
			  {
			    this.getSmdsub().getSmd().setOwnref(line.substring(pos).trim());
			  }
			  log.debug("{}","smd ownref:"+this.getSmdsub().getSmd().getOwnref());
			  log.debug("{}","smd othref:"+this.getSmdsub().getSmd().getOthref());
			  this.getSmdsub().getSmd().setCur(this.getSmtsub().getSmt().getCur());
			  this.getSmdsub().getSmd().setSmtinr(this.getSmtsub().getSmt().getInr());
			  this.getSmdsub().getSmd().setSmtno(this.getSmtsub().getSmt().getSmtno());
			  this.getSmdsub().getSmd().setSmtref(this.getSmtsub().getSmt().getRef());
			  this.getSmdsub().getSmd().setSmpinr(this.getSmpsub().getSmp().getInr());
			  ///recdat typ acc这3字段虽然是monly的，但是因为在创建DOT的时候要使用，所以将他们赋值
			  //$smdsub\\smd\\recdat=$smtsub\\smt\\recdat;
			  this.getSmdsub().getSmd().setRecdat(this.getMpi().getGendat());
			  this.getSmdsub().getSmd().setTyp(this.getSmtsub().getSmt().getTyp());
			  this.getSmdsub().getSmd().setAct(this.getSmtsub().getSmt().getAcc());
			  //log.debug("10"+$smdsub\\smd\\smtno);
			  //获取
			  //log.debug(dcflg);
			  if("C".equals(dcflg)||"RD".equals(dcflg))
			  {
			    all = all.add(amt);
			    //log.trace(" +"+amt);
			  }
			  else if("D".equals(dcflg)||"RC".equals(dcflg))
			  {
			    all=all.subtract(amt);
			    //log.trace(" -"+amt);
			  }
			  //log.debug("all="+all);
			  //$smdsub\\lst.add($smdsub\\smd); 直接add不能加进去
			  Platform.sdbCounter(this.getSmdsub().getSmd());
			  Platform.mdbInsert(this.getSmdsub().getSmd());
			  /* Argument<String> acur = new Argument<String>("cur",$smdsub\\smd\\cur.getValue());
			    Argument<BigDecimal> aamt = new Argument<BigDecimal>("amt",$smdsub\\smd\\amt.getValue());
			    Argument<Date> avaldat = new Argument<Date>("valdat",$smdsub\\smd\\valdat.getValue());
			    Argument<String> adcflg = new Argument<String>("dcflg",$smdsub\\smd\\dcflg.getValue());
			    Argument<String> aownref = new Argument<String>("ownref",$smdsub\\smd\\ownref.getValue());
			    Argument<String> aothref = new Argument<String>("othref",$smdsub\\smd\\othref.getValue());
			    Argument<String> ainr = new Argument<String>("inr",$smdsub\\smd\\inr.getValue());*/
			  //$smdsub\\lst.add(ainr,acur,aamt,avaldat,adcflg,aownref,aothref);
			  this.getSmdsub().getLst().add((IDatafield[])this.getSmdsub().getSmd().getDatafields().toArray(new IDatafield[0]));
			}
			log.debug("{}","all"+all);
			/// 如果重页状态不追加其他错误状态
			this.getSmpsub().getSmp().setPagamt(all);
			String errtyp = this.getSmpsub().getSmp().getErrtyp();
			if(errtyp!=null && (errtyp.contains("1") || errtyp.contains("2")))
			  return;
			///  根据明细总额 $pagamt校验页内试算平衡
			BigDecimal begamt = this.getSmpsub().getSmp().getNewbegamt();
			BigDecimal endamt = this.getSmpsub().getSmp().getNewendamt();
			if(endamt.subtract(begamt).compareTo(all)!=0)
			{
			  Platform.addError(MdaDriver.getDatafield(this.getSmpsub().getSmp(),"errtyp"),"6");// 页内试算不平衡
			  log.error("{}","页内试算不平衡");
			}
			else
			{
			  Platform.removeError(MdaDriver.getDatafield(this.getSmpsub().getSmp(),"errtyp"),"6");
			}
		}
        /**
        *   source:smtsvr.@0007.script
        */
        public String[] getNormalSmpinr(String smtinr,String exceptsmpinr)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			if(MdaUtils.isEmpty(smtinr))
			  return null;
			String sql = " SELECT inr FROM smp WHERE smtinr='"+smtinr+"' AND sta='0'";
			if(!MdaUtils.isEmpty(exceptsmpinr))
			  sql = sql+" AND inr !='"+exceptsmpinr+"'";
			log.debug("{}","sql = "+sql);
			Platform.mdbExecuteSQL(sql);
			Argument<String> inr = new Argument<String>("inr");
			List<String> list = new ArrayList<String>();
			while(true)
			{
			  Platform.mdbFetchFields(inr);
			  if(ctx.getErrorCode()==NO_ERROR)
			  {
			    list.add(inr.value);
			  }
			  else
			    break;
			}
			Platform.mdbCloseCursor();
			Object[] objs = list.toArray();
			String[] inrs =  new String[objs.length];
			for(int i=0;i<objs.length;i++)
			{
			  inrs[i] = objs[i].toString();
			}
			return inrs;
		}
        /**
        *   source:smtsvr.@0008.script
        */
        public void updateActAndGenerOrder()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			//更新账户余额，且可能生成划款指令
			if("msgimp".equals(this.getSmhcrefrm()))
			{
			  log.debug("{}","历史950报文导入不更新账户余额和生成划款指令");
			  return ;
			}
			String actbnkinr = this.getSmtsub().getSmt().getAccpty();
			String cur = this.getSmpsub().getSmp().getCur();
			log.debug("{}","进入更新账户余额，且判断是否生成划款指令");
			log.debug("{}","actbnkinr:"+actbnkinr);
			log.debug("{}","actbic:"+this.getSmtsub().getSmt().getBic());
			log.debug("{}","cur:"+cur);
			Platform.mdbRead(this.getSmpsub().getAct(),new Argument("actbnkinr",actbnkinr),new Argument("cur",cur));
			if(!MdaUtils.isEmpty(this.getSmpsub().getAct().getInr()))
			{
			  log.debug("{}","维护了该账户");
			  BigDecimal curAmt = this.getSmpsub().getSmp().getEndamt();
			  this.getSmpsub().getAct().setBln(curAmt);
			  Platform.mdbUpdate(this.getSmpsub().getAct());
			  if("A".equals(this.getSmpsub().getAct().getActcls()))
			  {
			    log.debug("{}","账户本身为主账户，不用判断上下限,inr： "+this.getSmpsub().getAct().getInr());
			    return ;
			  }
			  log.trace("{}","当前账户余额："+curAmt);
			  log.trace("{}","当前账户上限："+this.getSmpsub().getAct().getToplit());
			  log.trace("{}","当前账户下限："+this.getSmpsub().getAct().getLowlit());
			  if(curAmt.compareTo(this.getSmpsub().getAct().getLowlit()) < 0)
			  {
			    //低于下限，生成划款指令，该 账户设为汇入行
			    MdaUtils.clear(this.getSmpsub().getOpty());
			    Platform.mdbRead(this.getSmpsub().getOpty(),new Argument("inr",actbnkinr));
			    log.debug("{}","读取该账户对应的账户行："+this.getSmpsub().getOpty().getExtkey());
			    if(MdaUtils.isEmpty(this.getSmpsub().getOpty().getInr()))
			      return ;
			    this.getSmpsub().getHkz().setInswif( this.getSmpsub().getOpty().getExtkey());
			    this.getSmpsub().getHkz().setIegnam( this.getSmpsub().getOpty().getEngnam());
			    this.getSmpsub().getHkz().setIchnnam( this.getSmpsub().getOpty().getChnnam());
			    this.getSmpsub().getHkz().setInnact( this.getSmpsub().getAct().getActid());
			    this.getSmpsub().getHkz().setAmt( this.getSmpsub().getAct().getLowlit().subtract(curAmt));
			    log.debug("{}","划款指令币种："+cur);
			    this.getSmpsub().getHkz().setCur( cur);
			    this.getSmpsub().getHkz().setSta( "0");
			    //找到主账户，设置为汇出行
			    Platform.mdbRead(this.getSmpsub().getMat(),new Argument("cur",cur),new Argument("actcls","A"));
			    if(MdaUtils.isEmpty(this.getSmpsub().getMat().getInr()))
			    {
			      log.debug("{}","没有找到该币种的主账户") ;
			      return;//没有找到主账户
			    }
			    Platform.mdbRead(this.getSmpsub().getPty(),new Argument("inr",this.getSmpsub().getMat().getActbnkinr()));
			    this.getSmpsub().getHkz().setOuswif( this.getSmpsub().getPty().getExtkey());
			    this.getSmpsub().getHkz().setOegnam( this.getSmpsub().getPty().getEngnam());
			    this.getSmpsub().getHkz().setOchnam( this.getSmpsub().getPty().getChnnam());
			    this.getSmpsub().getHkz().setCredat( MdaUtils.today());
			    this.getSmpsub().getHkz().setOutact( this.getSmpsub().getMat().getActid());
			    Date vd = MdaUtils.parseDate(Platform.getAccountDate(),"yyyyMMdd");
			    this.getSmpsub().getHkz().setValdat( vd);

			    //关联所属实体
			    ctx.absGet(Sysmod.class).setEtyAndPty(this.getSmpsub().getHkz(),this.getSmtsub().getSmt());

			    Platform.sdbInsert(this.getSmpsub().getHkz());
			    log.debug("{}","生成了一笔从主账户划出的划款指令");
			  }
			  else if(curAmt.compareTo(this.getSmpsub().getAct().getToplit()) > 0)
			  {
			    //高于上线，生成划款指令，找到主账户，主账户为汇入行
			    Platform.mdbRead(this.getSmpsub().getMat(),new Argument("cur",cur),new Argument("actcls","A"));
			    if(MdaUtils.isEmpty(this.getSmpsub().getMat().getInr()))
			    {
			      log.debug("{}","没有找到该币种的主账户");
			      return;//没有找到主账户
			    }
			    Platform.mdbRead(this.getSmpsub().getPty(),new Argument("inr",this.getSmpsub().getMat().getActbnkinr()));
			    this.getSmpsub().getHkz().setInswif( this.getSmpsub().getPty().getExtkey());
			    this.getSmpsub().getHkz().setIegnam( this.getSmpsub().getPty().getEngnam());
			    this.getSmpsub().getHkz().setIchnnam( this.getSmpsub().getPty().getChnnam());
			    this.getSmpsub().getHkz().setAmt( curAmt.subtract(this.getSmpsub().getAct().getToplit()));
			    this.getSmpsub().getHkz().setSta( "0");
			    this.getSmpsub().getHkz().setCur( cur);
			    this.getSmpsub().getHkz().setInnact( this.getSmpsub().getMat().getActid());
			    log.debug("{}","划款指令币种："+cur);
			    //该账户为汇出行
			    Platform.mdbRead(this.getSmpsub().getOpty(),new Argument("inr",actbnkinr));
			    log.debug("{}","读取该账户对应的账户行："+this.getSmpsub().getOpty().getExtkey());
			    if(MdaUtils.isEmpty(this.getSmpsub().getOpty().getInr()))
			      return ;
			    this.getSmpsub().getHkz().setOuswif( this.getSmpsub().getOpty().getExtkey());
			    this.getSmpsub().getHkz().setOegnam( this.getSmpsub().getOpty().getEngnam());
			    this.getSmpsub().getHkz().setOchnam( this.getSmpsub().getOpty().getChnnam());
			    this.getSmpsub().getHkz().setCredat( MdaUtils.today());
			    this.getSmpsub().getHkz().setOutact( this.getSmpsub().getAct().getActid());
			    Date vd = MdaUtils.parseDate(Platform.getAccountDate(),"yyyyMMdd");
			    this.getSmpsub().getHkz().setValdat( vd);

			    //关联所属实体
			    ctx.absGet(Sysmod.class).setEtyAndPty(this.getSmpsub().getHkz(),this.getSmtsub().getSmt());

			    Platform.sdbInsert(this.getSmpsub().getHkz());
			    log.debug("{}","生成了一笔划入主账户的划款指令");
			  }
			}
		}





        public void setMpiHolder(ModuleHolder<Mpi> mpi)
        {
            this.mpi = mpi;
        }

        public Mpi getMpi()
        {
           return this.mpi.get();
        }











}