package com.ceb.gjjs.mda.manager.module;

import java.util.*;
import java.math.BigDecimal;
import com.brilliance.mda.runtime.mda.*;
import com.brilliance.mda.runtime.mda.impl.*;
import com.brilliance.mda.runtime.annotation.*;
import com.brilliance.mda.runtime.mda.driver.*;
import static com.brilliance.mda.runtime.mda.Constants.*;
import com.brilliance.mda.runtime.mda.util.*;
import com.fasterxml.jackson.annotation.*;
import java.util.regex.*;
import lombok.Getter;
import lombok.Setter;
import com.ceb.gjjs.mda.bo.Wfs;
import com.ceb.gjjs.mda.bo.Sro;
import com.ceb.gjjs.mda.global.Platform;
import com.ceb.gjjs.mda.bo.Wfe;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
* 
*/
@JsonIgnoreProperties(ignoreUnknown = true)
@Getter
@Setter
public class Wfmmod extends AbstractModule{
		private static Logger log = LoggerFactory.getLogger(Wfmmod.class);



	    @BDGetter
		private Wfs wfs;

	    @BDGetter
		private Wfe wfe;

		@BDGetter
		private ModuleList<Sro> srolst;


		public Wfmmod()
        {
            super();
            setArgument();
        }

        public Wfmmod(String name,IModule parent)
        {
            super(name,parent);
            setArgument();
        }

        public void setArgument()
        {
        }




        /**
        *   source:wfmmod.@0000.script
        */
        public void addWfe(String svr,String sta)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			this.getWfe().setWfsinr(this.getWfs().getInr());
			this.getWfe().setSrv(svr);
			this.getWfe().setSta(sta);
			this.getWfe().setDattim(new Date());
			this.getWfe().setManflg("0");
			this.getWfe().setTardattim(new Date());
			Platform.sdbCounter(this.getWfe());
			Platform.mdbInsert(this.getWfe());
			MdaUtils.clear(this.getWfe());
		}
        /**
        *   source:wfmmod.@0001.script
        */
        public void loadSro()
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  *	加载工作流服务及顺序
			  - 无
			  @ 无
			  */
			log.debug("{}","loadSro");
			MdaUtils.clear(this.getSrolst());
			Argument<BigDecimal> args = new Argument<BigDecimal>("lev",ASC);
			Platform.mdbReadset(this.getSrolst(),args);
			log.debug("{}","db read sro with errorcode:" + ctx.getErrorCode());
		}
        /**
        *   source:wfmmod.@0002.script
        */
        public int getCurrentServiceLevel(String srvName)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 获取当前服务级别
			  -	srvName 当前服务名称
			  @ int 当前服务级别
			  */
			log.debug("{}","getCurrentServiceLevel");
			int serviceLevel = -1;
			Argument<BigDecimal> arg = new Argument<BigDecimal>("lev");
			String sqlQuery = "SELECT LEV FROM SRO WHERE SRV = ? ";
			Platform.mdbExecuteSQL(sqlQuery,srvName);
			Platform.mdbFetchFields(arg);
			Platform.mdbCloseCursor();
			if(arg.value != null)
			  serviceLevel = arg.value.intValue();
			log.debug("{}","getCurrentServiceLevel:" + serviceLevel + " with errorcode:" + ctx.getErrorCode());
			return serviceLevel;
		}
        /**
        *   source:wfmmod.@0003.script
        */
        public int getCurrentServiceSubLevel(String srvName)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 获取当前服务子级别
			  -	srvName 当前服务名称
			  @ int 当前服务子级别
			  */
			log.debug("{}","getCurrentServiceSubLevel");
			int serviceSubLevel = -1;
			Argument<BigDecimal> arg = new Argument<BigDecimal>("sublev");
			String sqlQuery = "SELECT SUBLEV FROM SRO WHERE SRV = ? ";
			Platform.mdbExecuteSQL(sqlQuery,srvName);
			Platform.mdbFetchFields(arg);
			Platform.mdbCloseCursor();
			if(arg.value != null)
			  serviceSubLevel = arg.value.intValue();
			log.debug("{}","getCurrentServiceSubLevel:" + serviceSubLevel + " with errorcode:" + ctx.getErrorCode());
			return serviceSubLevel;
		}
        /**
        *   source:wfmmod.@0004.script
        */
        public String [] getNextServiceName(String srvName)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 获取当前服务下一服务名称
			  -	srvName 当前服务名称
			  @ int 当前服务下一服务名称
			  */
			log.debug("{}","getNextServiceName");
			String [] srvNames = null;
			MdaUtils.clear(this.getSrolst());
			Argument arg = new Argument();
			int nextLevel = getCurrentServiceNextLevel(srvName);
			arg = arg.and("lev",nextLevel);
			Platform.mdbReadset(this.getSrolst(),arg);
			if(this.getSrolst().size() > 0)
			{
			  srvNames = new String[this.getSrolst().size()];
			  int i=0;
			  for(com.ceb.gjjs.mda.bo.Sro sro : this.getSrolst())
			  {
			    srvNames[i] = sro.getSrv().toLowerCase();
			    log.debug("{}","load next service:" + srvNames[i]);
			    i++;
			  }
			}
			log.debug("{}","getNextServiceName with errorcode:" + ctx.getErrorCode());
			return srvNames;
		}
        /**
        *   source:wfmmod.@0005.script
        */
        public int getCurrentServiceNextLevel(String srvName)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 获取当前服务下一服务级别
			  -	srvName 当前服务名称
			  @ int 当前服务下一服务级别
			  */
			log.debug("{}","getCurrentServiceNextLevel");
			int nextLevel = -1;
			int currentLevel = getCurrentServiceLevel(srvName);
			Argument<BigDecimal> arg = new Argument<BigDecimal>("lev");
			String sqlQuery = "SELECT LEV FROM SRO WHERE LEV > ? ORDER BY LEV ASC ";
			Platform.mdbExecuteSQL(sqlQuery,currentLevel);
			Platform.mdbFetchFields(arg);
			Platform.mdbCloseCursor();
			if(arg.value != null)
			  nextLevel = arg.value.intValue();
			log.debug("{}","getCurrentServiceNextLevel:" + nextLevel + " with errorcode:" + ctx.getErrorCode());
			return nextLevel;
		}
        /**
        *   source:wfmmod.@0006.script
        */
        public boolean updateWfeState(IModule wfe,String newState)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  *	设置WFE模块的状态
			  - wfe 要修改状态的WFE模块
			  - newState 要设置的新状态
			  @ boolean 设置成否成功,成功返回true,否则返回false
			  */
			log.debug("{}","updateWfeState");
			IDatafield sta = MdaDriver.getDatafield(wfe,"sta");
			sta.setValue(newState);
			if("D".equals(newState) || "E".equals(newState))
			{
			  IDatafield<Date> tarDateTime = MdaDriver.getDatafield(wfe,"tardattim");
			  tarDateTime.setValue(MdaUtils.today());
			}
			Platform.mdbUpdate(wfe);
			int errorcode = ctx.getErrorCode();
			log.debug("{}","update wfe state to " + newState + " with errorcode:" + errorcode);
			if(errorcode == NO_ERROR)
			  return true;
			else
			  return false;
		}
        /**
        *   source:wfmmod.@0007.script
        */
        public boolean checkIsLastSameLevelService(String srvName)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 检测服务是否是当前级别的最后一个服务
			  - srvName 当前服务名称
			  @ boolean 是否是当前级别的最后一个服务,是返回true,否则返回false
			  */
			log.debug("{}","checkIsLastSameLevelService:" + srvName);
			int currentSubLevel = getCurrentServiceSubLevel(srvName);
			if(currentSubLevel == -1)
			  return true;
			else
			{
			  int nextSubLevel = getCurrentServiceNextSubLevel(srvName,currentSubLevel);
			  if(nextSubLevel == -1)
			    return true;
			  else
			    return false;
			}
		}
        /**
        *   source:wfmmod.@0008.script
        */
        public int getCurrentServiceNextSubLevel(String srvName,int currentSubLevel)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  * 获取当前服务下一子服务级别
			  -	srvName 当前服务名称
			  - currentSubLevel 当前子服务级别
			  @ int 当前服务下一子服务级别
			  */
			log.debug("{}","getCurrentServiceNextSubLevel");
			int nextSubLevel = -1;
			int currentLevel = getCurrentServiceLevel(srvName);
			Argument<BigDecimal> arg = new Argument<BigDecimal>("sublev");
			String sqlQuery = "SELECT SUBLEV FROM SRO WHERE LEV = ? AND SUBLEV > ? ORDER BY SUBLEV ASC ";
			Platform.mdbExecuteSQL(sqlQuery,currentLevel,currentSubLevel);
			Platform.mdbFetchFields(arg);
			Platform.mdbCloseCursor();
			if(arg.value != null)
			  nextSubLevel = arg.value.intValue();
			log.debug("{}","getCurrentServiceNextSubLevel:" + nextSubLevel + " with errorcode:" + ctx.getErrorCode());
			return nextSubLevel;
		}
        /**
        *   source:wfmmod.@0009.script
        */
        public void updateWfeDateTime(IModule wfe,String fieldName)
		{
			//取得上下文
			IContext ctx = MdaEnv.getContext();
			ctx.setErrorCode(NO_ERROR);
			/*
			  *	设置WFE模块时间字段值
			  - wfe 要修改状态的WFE模块
			  - fieldName 要设置的字段名称
			  @ 无
			  */
			log.debug("{}","updateWfeDateTime");
			IDatafield<Date> dateField = MdaDriver.getDatafield(wfe,fieldName);
			dateField.setValue(MdaUtils.today());
			Platform.mdbUpdate(wfe);
			int errorcode = ctx.getErrorCode();
			log.debug("{}","update wfe datetime with errorcode:" + errorcode);
		}




}